# -*- coding: utf-8 -*-

import system_tests


class PrettyPrintXmp(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/540"

    filename = "$data_path/exiv2-bug540.jpg"
    commands = ["$exiv2 -u -px $filename"]
    stdout = ["""Xmp.dc.creator                               XmpSeq      1  Ian Britton
Xmp.dc.description                           LangAlt     1  lang="x-default" Communications
Xmp.dc.rights                                LangAlt     1  lang="x-default" ian Britton - FreeFoto.com
Xmp.dc.source                                XmpText    12  FreeFoto.com
Xmp.dc.subject                               XmpBag      1  Communications
Xmp.dc.title                                 LangAlt     1  lang="x-default" Communications
Xmp.exif.ApertureValue                       XmpText     3  F16
Xmp.exif.BrightnessValue                     XmpText     8  0.260156
Xmp.exif.ColorSpace                          XmpText     1  sRGB
Xmp.exif.DateTimeOriginal                    XmpText    20  2002:07:13 15:58:28
Xmp.exif.ExifVersion                         XmpText     4  2.00
Xmp.exif.ExposureBiasValue                   XmpText     6  -13/20 EV
Xmp.exif.ExposureProgram                     XmpText     1  Shutter priority
Xmp.exif.FNumber                             XmpText     3  F0.6
Xmp.exif.FileSource                          XmpText     1  (0)
Xmp.exif.FlashpixVersion                     XmpText     4  1.00
Xmp.exif.FocalLength                         XmpText     3  0.0 mm
Xmp.exif.FocalPlaneResolutionUnit            XmpText     1  inch
Xmp.exif.FocalPlaneXResolution               XmpText     8  12.0508
Xmp.exif.FocalPlaneYResolution               XmpText     8  12.0508
Xmp.exif.GPSLatitude                         XmpText    13  54,59.380000N
Xmp.exif.GPSLongitude                        XmpText    12  1,54.850000W
Xmp.exif.GPSMapDatum                         XmpText     5  WGS84
Xmp.exif.GPSTimeStamp                        XmpText    20  2002:07:13 14:58:24
Xmp.exif.GPSVersionID                        XmpText     7  2.0.0.0
Xmp.exif.ISOSpeedRatings                     XmpSeq      1  0
Xmp.exif.MeteringMode                        XmpText     1  Multi-segment
Xmp.exif.PixelXDimension                     XmpText     4  2400
Xmp.exif.PixelYDimension                     XmpText     4  1600
Xmp.exif.SceneType                           XmpText     1  (0)
Xmp.exif.SensingMethod                       XmpText     1  One-chip color area
Xmp.exif.ShutterSpeedValue                   XmpText    10  1/724 s
Xmp.pdf.Keywords                             XmpText    14  Communications
Xmp.photoshop.AuthorsPosition                XmpText    12  Photographer
Xmp.photoshop.CaptionWriter                  XmpText    11  Ian Britton
Xmp.photoshop.Category                       XmpText     3  BUS
Xmp.photoshop.City                           XmpText     1   
Xmp.photoshop.Country                        XmpText    14  Ubited Kingdom
Xmp.photoshop.Credit                         XmpText    11  Ian Britton
Xmp.photoshop.DateCreated                    XmpText    10  2002-06-20
Xmp.photoshop.Headline                       XmpText    14  Communications
Xmp.photoshop.State                          XmpText     1   
Xmp.photoshop.SupplementalCategories         XmpBag      1  Communications
Xmp.photoshop.Urgency                        XmpText     1  5
Xmp.tiff.Artist                              XmpText    11  Ian Britton
Xmp.tiff.BitsPerSample                       XmpSeq      1  8
Xmp.tiff.Compression                         XmpText     1  6
Xmp.tiff.Copyright                           LangAlt     1  lang="x-default" ian Britton - FreeFoto.com
Xmp.tiff.ImageDescription                    LangAlt     1  lang="x-default" Communications
Xmp.tiff.ImageLength                         XmpText     3  400
Xmp.tiff.ImageWidth                          XmpText     3  600
Xmp.tiff.Make                                XmpText     8  FUJIFILM
Xmp.tiff.Model                               XmpText    12  FinePixS1Pro
Xmp.tiff.Orientation                         XmpText     1  top, left
Xmp.tiff.ResolutionUnit                      XmpText     1  inch
Xmp.tiff.Software                            XmpText    19  Adobe Photoshop 7.0
Xmp.tiff.XResolution                         XmpText     5  300
Xmp.tiff.YCbCrPositioning                    XmpText     1  Co-sited
Xmp.tiff.YResolution                         XmpText     5  300
Xmp.xmp.CreateDate                           XmpText    20  2002-07-13T15:58:28Z
Xmp.xmp.ModifyDate                           XmpText    20  2002-07-19T13:28:10Z
Xmp.xmpBJ.JobRef                             XmpText     0  type="Bag"
Xmp.xmpBJ.JobRef[1]                          XmpText     0  type="Struct"
Xmp.xmpBJ.JobRef[1]/stJob:name               XmpText    12  Photographer
Xmp.xmpMM.DocumentID                         XmpText    58  adobe:docid:photoshop:84d4dba8-9b11-11d6-895d-c4d063a70fb0
Xmp.xmpRights.Marked                         XmpText     4  True
Xmp.xmpRights.WebStatement                   XmpText    16  www.freefoto.com
"""]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path

class Test_issue_1011(metaclass=CaseMeta):

    filename = path("$data_path/Jp2Image_readMetadata_loop.poc")
    commands = ["$exiv2 " + filename]
    stdout   = [""]
    stderr   = ["""$exiv2_exception_message """ + filename + """:
$kerCorruptedMetadata
"""]
    retval = [1]


# -*- coding: utf-8 -*-

import system_tests


class TestCvePoC(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/140"

    filename = "$data_path/006-heap-buffer-over"
    commands = ["$exiv2 $filename"]
    stdout = [
        """File name       : $filename
File size       : 309965 Bytes
MIME type       : image/tiff
Image size      : 17 x 12305
Thumbnail       : None
Camera make     : 
Camera model    : 
Image timestamp : 
File number     : 
Exposure time   : 
Aperture        : 
Exposure bias   : 
Flash           : 
Flash bias      : 
Focal length    : 
Subject distance: 
ISO speed       : 
Exposure mode   : 
Metering mode   : 
Macro mode      : 
Image quality   : 
White balance   : 
Copyright       : 
Exif comment    : 

"""
    ]
    stderr = ["ignored"]
    retval = [0]

    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path

class Sony2FpTest(metaclass=CaseMeta):

    filename = path("$data_path/exiv2-pr906.exv")
    commands = ["$exiv2 -pa --grep Sony2Fp $filename"]

    stdout = ["""Exif.Sony2Fp.AmbientTemperature              SByte       1  19
Exif.Sony2Fp.FocusMode                       Byte        1  6
Exif.Sony2Fp.AFAreaMode                      Byte        1  12
Exif.Sony2Fp.FocusPosition2                  Byte        1  140
"""
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests


@system_tests.DeleteFiles("$filename_xmp")
class CheckXmpData(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1229"
    filename = "$data_path/exiv2-bug1229.jpg"
    filename_xmp = "$data_path/exiv2-bug1229.xmp"

    commands = ["""$exiv2 -eX $filename"""]

    stdout = [""]
    stderr = [""]
    retval = [0]

    def post_tests_hook(self):
        with open(self.filename_xmp, "r", encoding='utf-8') as xmp:
            content = xmp.read(-1)

        expected =  """<?xpacket begin="﻿" id="W5M0MpCehiHzreSzNTczkc9d"?>
<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="XMP Core 4.4.0-Exiv2">
 <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <rdf:Description rdf:about=""
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:xmpMM="http://ns.adobe.com/xap/1.0/mm/"
   xmpMM:InstanceID="uuid:e61b1e9d-b8e4-6a17-a443-74b51849baaf">
   <dc:subject>
    <rdf:Bag>
     <rdf:li>Celebs: JEnna Marbles</rdf:li>
    </rdf:Bag>
   </dc:subject>
  </rdf:Description>
 </rdf:RDF>
</x:xmpmeta>
<?xpacket end="w"?>"""

        self.assertMultiLineEqual(expected, content)



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyFiles, path


@CopyFiles("$data_path/mini9.tif")
class UseNonIntrusiveWriting(metaclass=CaseMeta):

    url = "http://dev.exiv2.org/issues/831"

    filename = path("$data_path/mini9_copy.tif")

    commands = [
        """$exiv2 -v -Qd -M"set Exif.Image.ImageDescription Just GIMP" $filename""",
        "$exiv2 -v -pa $filename"
    ]

    stdout = [
        """File 1/1: $filename
Set Exif.Image.ImageDescription "Just GIMP" (Ascii)
""",
        """File 1/1: $filename
Exif.Image.NewSubfileType                    Long        1  Primary image
Exif.Image.ImageWidth                        Short       1  9
Exif.Image.ImageLength                       Short       1  9
Exif.Image.BitsPerSample                     Short       3  8 8 8
Exif.Image.Compression                       Short       1  Uncompressed
Exif.Image.PhotometricInterpretation         Short       1  RGB
Exif.Image.DocumentName                      Ascii      24  /home/ahuggel/mini9.tif
Exif.Image.ImageDescription                  Ascii      10  Just GIMP
Exif.Image.StripOffsets                      Long        1  8
Exif.Image.Orientation                       Short       1  top, left
Exif.Image.SamplesPerPixel                   Short       1  3
Exif.Image.RowsPerStrip                      Short       1  64
Exif.Image.StripByteCounts                   Long        1  243
Exif.Image.XResolution                       Rational    1  72
Exif.Image.YResolution                       Rational    1  72
Exif.Image.PlanarConfiguration               Short       1  Chunky
Exif.Image.ResolutionUnit                    Short       1  inch
"""
    ]
    stderr = [
        """Info: Write strategy: Non-intrusive
""",
        """$filename: No IPTC data found in the file
$filename: No XMP data found in the file
"""
    ]
    retval = [0] * 2



# -*- coding: utf-8 -*-

import system_tests


class TestCvePoC(metaclass=system_tests.CaseMeta):

    url = [
        "https://github.com/Exiv2/exiv2/issues/139",
        "https://bugzilla.redhat.com/show_bug.cgi?id=1494787"
    ]

    filename = "$data_path/009-stack-over"
    commands = ["$exiv2 $filename"]
    stdout = ["""File name       : $filename
File size       : 340 Bytes
MIME type       : image/tiff
Image size      : 0 x 0
Thumbnail       : None
Camera make     : 
Camera model    : 
Image timestamp : 
File number     : 
Exposure time   : 
Aperture        : 
Exposure bias   : 
Flash           : 
Flash bias      : 
Focal length    : 
Subject distance: 
ISO speed       : 
Exposure mode   : 
Metering mode   : 
Macro mode      : 
Image quality   : 
White balance   : 
Copyright       : 
Exif comment    : 

"""
    ]
    stderr = [
        """Error: Directory Image: Next pointer is out of bounds; ignored.
Error: Offset of directory Image, entry 0x00fe is out of bounds: Offset = 0x00000000; truncating the entry
Error: Directory Image, entry 0x0100 has invalid size 1935897193*2; skipping entry.
Warning: Directory Image, entry 0x303e has unknown Exif (TIFF) type 12320; setting type size 1.
Error: Offset of directory Image, entry 0x0116 is out of bounds: Offset = 0x0011302a; truncating the entry
Warning: Directory Image, entry 0x0111: Strip 0 is outside of the data area; ignored.
"""
    ]
    retval = [0]



# -*- coding: utf-8 -*-

import itertools

import system_tests


class PanasonicManometerTag(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/825"

    filenames = [
        "$data_path/exiv2-bug825{!s}.exv".format(char)
        for char in ["a", "b"]
    ]

    commands = list(itertools.chain(
        *([
            "$exiv2 -pv --grep mano/i " + filename,
            "$exiv2 -pa --grep mano/i " + filename
        ] for filename in filenames)
    ))

    stdout = [
        """0x0086 Panasonic    ManometerPressure           Short       1  65535
""",
        """Exif.Panasonic.ManometerPressure             Short       1  infinite
""",
        """0x0086 Panasonic    ManometerPressure           Short       1  1007
""",
        """Exif.Panasonic.ManometerPressure             Short       1  1007 hPa
"""
    ]
    stderr = [""] * 4
    retval = [0] * 4



# -*- coding: utf-8 -*-

import system_tests


@system_tests.CopyFiles("$data_path/exiv2-empty.jpg")
class UnicodeEncodingOfExifUserCommentTag(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/662"

    filename = "$data_path/exiv2-empty_copy.jpg"
    commands = [
        """$exiv2 -u -M"set Exif.Photo.UserComment charset=Ascii An ascii comment" $filename""",
        "$exiv2 -u -PEnh $filename",

        """$exiv2 -u -M"set Exif.Photo.UserComment charset=Ascii A\\nnewline" $filename""",
        "$exiv2 -u -PEnh $filename",

        """$exiv2 -u -M"set Exif.Photo.UserComment charset=Unicode A Unicode comment" $filename""",
        "$exiv2 -u -PEnh $filename",

        """$exiv2 -u -M"set Exif.Photo.UserComment charset=Unicode \\u01c4" $filename""",
        "$exiv2 -u -PEnh $filename",

        """$exiv2 -u -M"set Exif.Photo.UserComment charset=Unicode A\\u01c4C" $filename""",
        "$exiv2 -u -PEnh $filename",

        """$exiv2 -u -M"set Exif.Photo.UserComment charset=Unicode With\\nNewline" $filename""",
        "$exiv2 -u -PEnh $filename",

        """$exiv2 -u -M"set Exif.Photo.UserComment charset=Unicode With\\tTab" $filename""",
        "$exiv2 -u -PEnh $filename",

        #Test invalid escape sequences
        """$exiv2 -u -M"set Exif.Photo.UserComment charset=Unicode \\ugggg" $filename""",
        "$exiv2 -u -PEnh $filename",
    ]
    stdout = [
        "",
        """ExifTag                    
  0000  1a 00 00 00                                      ....

UserComment                
  0000  41 53 43 49 49 00 00 00 41 6e 20 61 73 63 69 69  ASCII...An ascii
  0010  20 63 6f 6d 6d 65 6e 74                           comment

""",
        "",
        """ExifTag                    
  0000  1a 00 00 00                                      ....

UserComment                
  0000  41 53 43 49 49 00 00 00 41 0a 6e 65 77 6c 69 6e  ASCII...A.newlin
  0010  65                                               e

""",
        "",
        """ExifTag                    
  0000  1a 00 00 00                                      ....

UserComment                
  0000  55 4e 49 43 4f 44 45 00 41 00 20 00 55 00 6e 00  UNICODE.A. .U.n.
  0010  69 00 63 00 6f 00 64 00 65 00 20 00 63 00 6f 00  i.c.o.d.e. .c.o.
  0020  6d 00 6d 00 65 00 6e 00 74 00                    m.m.e.n.t.

""",
        "",

        """ExifTag                    
  0000  1a 00 00 00                                      ....

UserComment                
  0000  55 4e 49 43 4f 44 45 00 c4 01                    UNICODE...

""",
        "",
"""ExifTag                    
  0000  1a 00 00 00                                      ....

UserComment                
  0000  55 4e 49 43 4f 44 45 00 41 00 c4 01 43 00        UNICODE.A...C.

""",
        "",
        """ExifTag                    
  0000  1a 00 00 00                                      ....

UserComment                
  0000  55 4e 49 43 4f 44 45 00 57 00 69 00 74 00 68 00  UNICODE.W.i.t.h.
  0010  0a 00 4e 00 65 00 77 00 6c 00 69 00 6e 00 65 00  ..N.e.w.l.i.n.e.

""",
        "",
        """ExifTag                    
  0000  1a 00 00 00                                      ....

UserComment                
  0000  55 4e 49 43 4f 44 45 00 57 00 69 00 74 00 68 00  UNICODE.W.i.t.h.
  0010  09 00 54 00 61 00 62 00                          ..T.a.b.

""",
        "",
        """ExifTag                    
  0000  1a 00 00 00                                      ....

UserComment                
  0000  55 4e 49 43 4f 44 45 00 5c 00 75 00 67 00 67 00  UNICODE.\.u.g.g.
  0010  67 00 67 00                                      g.g.

"""
    ]
    stderr = [""] * 16
    retval = [0] * 16



# -*- coding: utf-8 -*-

import system_tests


class NikonLensId227(metaclass=system_tests.CaseMeta):

    filename = "$data_path/exiv2-bug1348.exv"
    commands = ["$exiv2 -pa --grep lensidnumber/i $filename"]

    stdout = ["""Exif.NikonLd3.LensIDNumber                   Byte        1  Tamron SP AF 150-600mm F/5-6.3 Di VC USD G2
"""
	]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path, check_no_ASAN_UBSAN_errors


class EmptyValueInCommandFile(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1099

    An empty value in the command file causes a std::out_of_range exception.
    """
    url = "https://github.com/Exiv2/exiv2/issues/1099"

    filename1 = path("$data_path/issue_1099_poc.txt")
    filename2 = path("$data_path/issue_1099_poc.bin")
    commands = ["$exiv2 -m $filename1 mo $filename2"]

    stderr = [
        """$filename1, line 1: Empty value for key `Exiff.LfkInfo.GPSDa'
exiv2: Error parsing -m option arguments
"""]
    retval = [1]

    def compare_stdout(self, i, command, got_stdout, expected_stdout):
        """ We don't care about the stdout, just don't crash """
        pass



# -*- coding: utf-8 -*-

import unittest
import os.path
import shutil

import system_tests


RESOURCE_FORK_EXISTS = os.path.exists(
    os.path.join(system_tests.data_path, "exiv2-bug836.eps/..namedfork/rsrc")
)


@unittest.skipUnless(RESOURCE_FORK_EXISTS,
                     "File system does not support resource forks")
@system_tests.CopyFiles("$data_path/exiv2-bug836.eps")
class WriteMetadataDestroysResourceForkOnMacOSXForBigFiles(
        metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/836"

    filename = "$data_path/exiv2-bug836_copy.eps"
    filename_orig = "$data_path/exiv2-bug836.eps"

    commands = [
        "$exiv2 -M'set Exif.Photo.UserComment Test' $filename",
    ]

    def setUp(self):
        """ Copy the rsrc file into the the resource fork """
        shutil.copyfile(
            self.filename_orig + ".rsrc", self.filename + "/..namedfork/rsrc"
        )

    def post_tests_hook(self):
        """ Check that the resource fork didn't change """
        with open(self.filename_orig + ".rsrc", "rb") as rsrc_orig:
            expected_resource_fork = rsrc_orig.read(-1)

        with open(self.filename + "/..namedfork/rsrc", "rb") as rsrc_f:
            got_resource_fork = rsrc_f.read(-1)

        self.assertEqual(expected_resource_fork, got_resource_fork)

    stdout = [""]
    stderr = [""]
    retval = [0]



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import itertools
import multiprocessing
import os
import shlex
import subprocess
import sys


def call_wrapper(*args, **kwargs):
    """
    Wrapper around subprocess.call which terminates the program on non-zero
    return value.
    """
    return_code = subprocess.call(*args, **kwargs)
    if return_code != 0:
        sys.exit(return_code)


def matrix_build(shared_libs, ccs, build_types, cmake_bin, cmake_options,
                 tests=True):

    NCPUS = multiprocessing.cpu_count()

    os.mkdir("build")

    for params in itertools.product(shared_libs, ccs, build_types):

        lib_type, cc, build_type = params

        cwd = os.path.abspath(
            os.path.join(
                "build",
                "_".join(
                    map(lambda p: str(p) if p is not None else "", params)
                )
            )
        )
        os.mkdir(cwd)

        cmake = "{cmake_bin} {!s} -DCMAKE_BUILD_TYPE={build_type} -DCMAKE_CXX_FLAGS=-Wno-deprecated " \
            "-DBUILD_SHARED_LIBS={lib_type} -DEXIV2_BUILD_UNIT_TESTS={tests} -DCMAKE_CXX_STANDARD=98 "\
            "../..".format(
                cmake_options, cmake_bin=cmake_bin, build_type=build_type,
                lib_type=lib_type, tests="ON" if tests else "OFF"
            )
        make = "make -j " + str(NCPUS)
        make_tests = "make tests"
        unit_test_binary = os.path.join(cwd, "bin", "unit_tests")

        # set compiler via environment only when requested
        env_copy = os.environ.copy()
        if cc is not None:
            cxx = {"gcc": "g++", "clang": "clang++"}[cc]
            env_copy["CC"] = cc
            env_copy["CXX"] = cxx

        # location of the binaries for the new test suite:
        env_copy["EXIV2_BINDIR"] = os.path.join(cwd, "bin")

        kwargs = {"env": env_copy, "cwd": cwd}

        def run(cmd):
            call_wrapper(shlex.split(cmd), **kwargs)

        run(cmake)
        run(make)
        if tests:
            run(make_tests)
            run(unit_test_binary)


if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser(
        description="Build and test exiv2 using a matrix of build switches")
    parser.add_argument(
        "--compilers",
        help="Compilers to be used to build exiv2 (when none ore specified, "
        "then the default compiler will be used)",
        nargs='*',
        default=["gcc", "clang"],
        type=str
    )
    parser.add_argument(
        "--shared-libs",
        help="Values for the -DBUILD_SHARED_LIBS option",
        nargs='+',
        default=["ON", "OFF"],
        type=str
    )
    parser.add_argument(
        "--build-types",
        help="Values for the -DCMAKE_BUILD_TYPE option",
        nargs='+',
        default=["Debug", "Release"],
        type=str
    )
    parser.add_argument(
        "--cmake-executable",
        help="alternative name or path for the cmake executable",
        nargs=1,
        default=['cmake'],
        type=str
    )
    parser.add_argument(
        "--without-tests",
        help="Skip building and running tests",
        action='store_true'
    )
    parser.add_argument(
        "--cmake-options",
        help="Additional flags for cmake",
        type=str,
        nargs='?',
        default="-DEXIV2_TEAM_EXTRA_WARNINGS=ON -DEXIV2_ENABLE_VIDEO=ON "
        "-DEXIV2_ENABLE_WEBREADY=ON -DEXIV2_BUILD_UNIT_TESTS=ON -DEXIV2_ENABLE_BMFF=ON "
        "-DBUILD_WITH_CCACHE=ON -DEXIV2_ENABLE_CURL=ON"
    )

    args = parser.parse_args()

    if len(args.compilers) == 0:
        args.compilers = [None]

    matrix_build(
        args.shared_libs, args.compilers, args.build_types,
        args.cmake_executable[0], args.cmake_options,
        not args.without_tests
    )



# -*- coding: utf-8 -*-

import system_tests


@system_tests.CopyFiles("$data_path/exiv2-bug784.jpg")
class WritingXmpLrHierarchicalSubjectWritesWrongDatatype(
        metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/784"

    filename = "$data_path/exiv2-bug784_copy.jpg"

    commands = [
        "$exiv2 -px $filename",
        """$exiv2 -M"add Xmp.lr.hierarchicalSubject root|1st|2nd|3rd|4th|5th" $filename""",
        "$exiv2 -px $filename",
    ]

    stdout = [
        """Xmp.dc.format                                XmpText    10  image/jpeg
Xmp.dc.rights                                LangAlt     1  lang="x-default" Public Domain. Do whatever you like with this image
Xmp.dc.subject                               XmpBag      6  1st, 2nd, next1, next2, root, root0
Xmp.xmpMM.DocumentID                         XmpText    32  004D48F936062EF5085A81BF96D4C494
Xmp.xmpMM.OriginalDocumentID                 XmpText    32  004D48F936062EF5085A81BF96D4C494
Xmp.xmpMM.InstanceID                         XmpText    44  xmp.iid:f74f0d02-e921-134e-8107-1dda17aad853
Xmp.xmpMM.History                            XmpText     0  type="Seq"
Xmp.xmpMM.History[1]                         XmpText     0  type="Struct"
Xmp.xmpMM.History[1]/stEvt:action            XmpText     5  saved
Xmp.xmpMM.History[1]/stEvt:instanceID        XmpText    44  xmp.iid:f74f0d02-e921-134e-8107-1dda17aad853
Xmp.xmpMM.History[1]/stEvt:when              XmpText    25  2015-03-24T20:35:55-05:00
Xmp.xmpMM.History[1]/stEvt:softwareAgent     XmpText    39  Adobe Photoshop Lightroom 4.4 (Windows)
Xmp.xmpMM.History[1]/stEvt:changed           XmpText     9  /metadata
Xmp.xmp.MetadataDate                         XmpText    25  2015-03-24T20:35:55-05:00
Xmp.crs.RawFileName                          XmpText    12  exiv2.lr.jpg
Xmp.lr.hierarchicalSubject                   XmpBag      2  root0|next1|next2, root|1st|2nd
""",
        "",
        """Xmp.dc.format                                XmpText    10  image/jpeg
Xmp.dc.rights                                LangAlt     1  lang="x-default" Public Domain. Do whatever you like with this image
Xmp.dc.subject                               XmpBag      6  1st, 2nd, next1, next2, root, root0
Xmp.xmpMM.DocumentID                         XmpText    32  004D48F936062EF5085A81BF96D4C494
Xmp.xmpMM.OriginalDocumentID                 XmpText    32  004D48F936062EF5085A81BF96D4C494
Xmp.xmpMM.InstanceID                         XmpText    44  xmp.iid:f74f0d02-e921-134e-8107-1dda17aad853
Xmp.xmpMM.History                            XmpText     0  type="Seq"
Xmp.xmpMM.History[1]                         XmpText     0  type="Struct"
Xmp.xmpMM.History[1]/stEvt:action            XmpText     5  saved
Xmp.xmpMM.History[1]/stEvt:instanceID        XmpText    44  xmp.iid:f74f0d02-e921-134e-8107-1dda17aad853
Xmp.xmpMM.History[1]/stEvt:when              XmpText    25  2015-03-24T20:35:55-05:00
Xmp.xmpMM.History[1]/stEvt:softwareAgent     XmpText    39  Adobe Photoshop Lightroom 4.4 (Windows)
Xmp.xmpMM.History[1]/stEvt:changed           XmpText     9  /metadata
Xmp.xmp.MetadataDate                         XmpText    25  2015-03-24T20:35:55-05:00
Xmp.crs.RawFileName                          XmpText    12  exiv2.lr.jpg
Xmp.lr.hierarchicalSubject                   XmpBag      1  root|1st|2nd|3rd|4th|5th
"""
    ]

    stderr = [""] * 3
    retval = [0] * 3



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path, check_no_ASAN_UBSAN_errors

class TiffMnEntryDoCountInvalidTiffType(metaclass=CaseMeta):
    """
    Test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1833
    """
    url = "https://github.com/Exiv2/exiv2/issues/1833"

    filename = path("$data_path/issue_1833_poc.jpg")
    commands = ["$exiv2 -pS $filename"]
    stderr = [""]
    retval = [0]

    compare_stdout = check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

import system_tests


class TestCvePoC(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/50"

    filename = "$data_path/POC3"
    commands = ["$exiv2 $filename"]
    stdout = [
        """File name       : $filename
File size       : 28 Bytes
MIME type       : image/tiff
Image size      : 0 x 0
"""
    ]
    stderr = [
        """Error: Directory Image, entry 0x0144 has invalid size 4294967295*8; skipping entry.
$filename: No Exif data found in the file
"""
    ]
    retval = ["$no_exif_data_found_retval"]



# -*- coding: utf-8 -*-

import system_tests
class issue_1504_metacopy(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1504"
    commands = ["$metacopy"
               ,"$metacopy -h"
               ]
    retval = [ 1,2] 
    stderr = [ """metacopy: Read and write files must be specified
""",""]
    stdout = ["""
Reads and writes raw metadata. Use -h option for help.
Usage: metacopy [-iecxaph] readfile writefile
""","""
Reads and writes raw metadata. Use -h option for help.
Usage: metacopy [-iecxaph] readfile writefile

Options:
   -i      Read Iptc data from readfile and write to writefile.
   -e      Read Exif data from readfile and write to writefile.
   -c      Read Jpeg comment from readfile and write to writefile.
   -x      Read XMP data from readfile and write to writefile.
   -a      Read all metadata from readfile and write to writefile.
   -p      Preserve existing metadata in writefile if not replaced.
   -h      Display this help and exit.

"""]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class MinoltaDivZero(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/security/advisories/GHSA-pvjp-m4f6-q984
    """
    url = "https://github.com/Exiv2/exiv2/security/advisories/GHSA-pvjp-m4f6-q984"

    filename = path("$data_path/issue_ghsa_pvjp_m4f6_q984_poc.exv")
    commands = ["$exiv2 -p t $filename"]
    stderr = ["""Error: Upper boundary of data for directory Image, entry 0x011a is out of bounds: Offset = 0x000000f2, size = 14155784, exceeds buffer size by 14110766 Bytes; truncating the entry
Error: Offset of directory Photo, entry 0x829a is out of bounds: Offset = 0x6d00035e; truncating the entry
Error: Upper boundary of data for directory Photo, entry 0x8822 is out of bounds: Offset = 0x00000003, size = 56834, exceeds buffer size by 11577 Bytes; truncating the entry
Error: Upper boundary of data for directory Photo, entry 0x8827 is out of bounds: Offset = 0x00000640, size = 1179650, exceeds buffer size by 1135990 Bytes; truncating the entry
Warning: Directory Photo, entry 0x8832 has unknown Exif (TIFF) type 49; setting type size 1.
Error: Offset of directory Sony2, entry 0x2006 is out of bounds: Offset = 0x00000000; truncating the entry
Warning: Directory Sony2, entry 0x20c1 has unknown Exif (TIFF) type 181; setting type size 1.
Error: Offset of directory Sony2, entry 0x2063 is out of bounds: Offset = 0x00000000; truncating the entry
Error: Offset of directory Sony2, entry 0x3000 is out of bounds: Offset = 0x0057097c; truncating the entry
Error: Offset of directory Sony2, entry 0x0115 is out of bounds: Offset = 0x00000000; truncating the entry
Error: Upper boundary of data for directory Sony2, entry 0x2013 is out of bounds: Offset = 0x00000002, size = 37486596, exceeds buffer size by 37441338 Bytes; truncating the entry
Warning: Directory Photo, entry 0xa003 has unknown Exif (TIFF) type 242; setting type size 1.
Warning: Directory Iop has an unexpected next pointer; ignored.
Warning: Directory Photo, entry 0xa402 has unknown Exif (TIFF) type 89; setting type size 1.
Error: Offset of directory Photo, entry 0xa402 is out of bounds: Offset = 0x00000000; truncating the entry
Error: Offset of directory Thumbnail, entry 0x0132 is out of bounds: Offset = 0xff00968b; truncating the entry
"""]
    retval = [0]

    def compare_stdout(self, i, command, got_stdout, expected_stdout):
        """ We don't care about the stdout, just don't crash """
        pass



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path
@CopyTmpFiles("$data_path/issue_ghsa_8949_hhfh_j7rj_poc.jp2","$data_path/issue_ghsa_8949_hhfh_j7rj_poc.exv")

class Jp2ImageEncodeJp2HeaderOutOfBoundsRead(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/security/advisories/GHSA-8949-hhfh-j7rj
    """
    url = "https://github.com/Exiv2/exiv2/security/advisories/GHSA-8949-hhfh-j7rj"

    filename1 = path("$tmp_path/issue_ghsa_8949_hhfh_j7rj_poc.jp2")
    filename2 = path("$tmp_path/issue_ghsa_8949_hhfh_j7rj_poc.exv")
    commands = ["$exiv2 in $filename1"]
    stdout = [""]
    stderr = [
"""Error: XMP Toolkit error 201: Error in XMLValidator
Warning: Failed to decode XMP metadata.
"""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests
import unittest
from tempfile import TemporaryDirectory
import shutil
import hashlib
import os

# test needs system_tests.BT.vv['enable_bmff']=1
bSkip=system_tests.BT.verbose_version().get('enable_bmff')!='1'

if bSkip:
    raise unittest.SkipTest('*** requires enable_bmff=1 ***')

file_basename = 'Canon-R6-pruned.CR3'
previews_expected = (
    ('Canon-R6-pruned-preview1.jpg', 'a182ef12ac883309b4dfc66b87eac1891286d3ae'),
    ('Canon-R6-pruned-preview2.jpg', '524a07f1797854e349ae140e2682ba37147fa6b2')
)

class issue_1893_cr3_preview(metaclass=system_tests.CaseMeta):
    """
    Check that THMB and PRVW images are extracted from Canon CR3 files
    """
    url = "https://github.com/Exiv2/exiv2/issues/1893"
    filename = "$data_path/" + file_basename
    commands=[] # see setUp()

    if bSkip:
        retval=[]
        stdin=[]
        stderr=[]
        stdout=[]
        print("*** test skipped.  requires enable_bmff=1***")
    else:
        retval = [ 0, 0]
        stderr = [ "",""]
        stdin  = [ "", ""]
        stdout = ["""Preview 1: image/jpeg, 160x120 pixels, 16005 bytes
Preview 2: image/jpeg, 1620x1080 pixels, 389450 bytes
""", ""]

    def post_tests_hook(self):
        if self.commands:
            for j, sha1 in previews_expected:
                p = os.path.join(self.preview_image_tmp_dir.name, j)
                self.assertTrue(os.path.isfile(p))
                h = hashlib.sha1(open(p, 'rb').read()).hexdigest()
                self.assertEqual(h, sha1)

    def setUp(self):
        if bSkip:
            return
        # Avoid polluting the test data directory with extracted previews
        self.preview_image_tmp_dir = TemporaryDirectory()
        shutil.copy(self.expand_variables(self.filename),
                    self.preview_image_tmp_dir.name)
        p = os.path.join(
            self.preview_image_tmp_dir.name,
            file_basename)
        self.commands = [
            self.expand_variables("$exiv2 -pp ") + p,
            self.expand_variables("$exiv2 -ep ") + p
            ]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class MrmImageLargeAllocation(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/pull/943
    """
    url = "https://github.com/Exiv2/exiv2/pull/943"

    filename1 = path("$data_path/issue_943_poc1.mrm")
    filename2 = path("$data_path/issue_943_poc2.mrm")
    commands = ["$exiv2 $filename1", "$exiv2 $filename2"]
    stdout = ["",""]
    stderr = [
"""Exiv2 exception in print action for file $filename1:
Failed to read image data
""",
"""Exiv2 exception in print action for file $filename2:
Failed to read image data
"""
]
    retval = [1,1]



# -*- coding: utf-8 -*-

import system_tests


class TestCvePoC(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/73"

    filename = "$data_path/02-Invalid-mem-def"
    commands = ["$exiv2 " + filename]
    stdout = [""]
    stderr = ["""$exiv2_exception_message """ + filename + """:
$kerCorruptedMetadata
"""]
    retval = [1]



#!/usr/bin/env python3

import sys
import csv

print("""<?xml version = '1.0'?>
<?xml-stylesheet type=\"text/xsl\" href=\"tags.xsl\"?>
<TAGLIST>
<HEADER>
<title>XMP tags defined in Exiv2</title>
<text>
<p>Some description</p>
<p>Click on a column header to sort the table.</p>
</text>
</HEADER>
<ROWSET>""")

row=0
data = sys.stdin.readlines()
print(data)
for line in csv.reader(data,quotechar='"',skipinitialspace=True):
    row=row+1
    print("   <ROW num=\"%d\">" % row) 
    print("      <tagname>"      + line[0] + "</tagname>")
    print("      <title>"        + line[1] + "</title>")
    print("      <xmpvaltype>"   + line[2] + "</xmpvaltype>")
    print("      <type>"         + line[3] + "</type>")
    print("      <category>"     + line[4] + "</category>")
    print("      <tagdesc>"      + line[5] + "</tagdesc>")
    print("   </ROW>")

print("</ROWSET>")
print("</TAGLIST>")




#!/usr/bin/env python3

import sys
import csv

print("""<?xml version = '1.0'?>
<?xml-stylesheet type=\"text/xsl\" href=\"tags.xsl\"?>
<TAGLIST>
<HEADER>
<title>XYZ MakerNote Tags defined in Exiv2</title>
<text>
<p>Tags found in the MakerNote of images taken with XYZ cameras. These tags 
are defined by Exiv2 in accordance with <a href=\"makernote.html#RX\">[X]</a>.</p>
<p>Click on a column header to sort the table.</p>
</text>
</HEADER>
<ROWSET>""")

row=0
data = sys.stdin.readlines()
for line in csv.reader(data,quotechar='"',skipinitialspace=True):
    row=row+1
    print("   <ROW num=\"%d\">" % row)
    print("      <tagname>" + line[0] + "</tagname>")
    print("      <tagdec>"  + line[1] + "</tagdec>")
    print("      <taghex>"  + line[2] + "</taghex>")
    print("      <ifd>"     + line[3] + "</ifd>")
    print("      <key>"     + line[4] + "</key>")
    print("      <type>"    + line[5] + "</type>")
    print("      <tagdesc>" + line[6] + "</tagdesc>")
    print("   </ROW>")

print("</ROWSET>")
print("</TAGLIST>")




from system_tests import CaseMeta, path


class OverreadInCiffDirectoryReadDirectory(metaclass=CaseMeta):

    filename = path("$data_path/POC-file_issue_1019")
    commands = ["$exiv2 -pv $filename"]
    stdout = [""]
    stderr = [
        """$exiv2_exception_message $filename:
$kerOffsetOutOfRange
"""
    ]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import DeleteFiles, CopyFiles, CaseMeta, path


@DeleteFiles("$xmpname")
@CopyFiles("$data_path/exiv2-empty.jpg")
class AdobeXmpNamespace(metaclass=CaseMeta):

    url = "http://dev.exiv2.org/issues/751"

    filename = path("$data_path/exiv2-empty_copy.jpg")
    xmpname = path("$data_path/exiv2-empty_copy.xmp")

    commands = [
        """$exiv2 -v -M"reg imageapp orig/" -M "set Xmp.imageapp.uuid abcd" $filename""",
        "$exiv2 -f -eX $filename",
        """$exiv2 -v -M"reg imageapp dest/" -M "set Xmp.imageapp.uuid abcd" $filename""",
        "$exiv2 -f -eX $filename",
    ]

    def post_command_hook(self, i, command):
        def read_xmpfile():
            with open(self.xmpname, "r", encoding='utf-8') as xmp:
                return xmp.read(-1)

        if i == 2 or i == 4:
            self.assertMultiLineEqual(self.xmp_packets[i//2 - 1], read_xmpfile())


    xmp_packets = [
        """<?xpacket begin="\ufeff" id="W5M0MpCehiHzreSzNTczkc9d"?>
<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="XMP Core 4.4.0-Exiv2">
 <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <rdf:Description rdf:about=""
    xmlns:imageapp="orig/"
   imageapp:uuid="abcd"/>
 </rdf:RDF>
</x:xmpmeta>
<?xpacket end="w"?>""",
        """<?xpacket begin="\ufeff" id="W5M0MpCehiHzreSzNTczkc9d"?>
<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="XMP Core 4.4.0-Exiv2">
 <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <rdf:Description rdf:about=""
    xmlns:imageapp="orig/"
   imageapp:uuid="abcd"/>
 </rdf:RDF>
</x:xmpmeta>
<?xpacket end="w"?>"""
    ]

    stdout = [
        """File 1/1: $filename
Reg imageapp="orig/"
Set Xmp.imageapp.uuid "abcd" (XmpText)
""",
        "",
        """File 1/1: $filename
Reg imageapp="dest/"
Set Xmp.imageapp.uuid "abcd" (XmpText)
""",
        "",
    ]
    stderr = [
        "",
        "",
        """Warning: Updating namespace URI for imageapp from orig/ to dest/
""",
        """Warning: Updating namespace URI for imageapp from dest/ to orig/
""",
    ]
    retval = [0] * 4



import system_tests


class BigTiffImageRecursionStackExhaustion(
        metaclass=system_tests.CaseMeta):
    """
    src/bigtiffimage.cpp is longer in the code base
    however, let's retain this test as support for BigTiff will
    be developed and the malicious test file may stress the bigtiff parser.
    
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/712

    A malicious input file can cause BigTiffImage::printIFD() to
    recurse arbitrarily deep, causing a crash due to stack exhaustion.

    The bug only existed in the -pR mode, which is now only enabled
    in debug builds.
    """
    url = "https://github.com/Exiv2/exiv2/issues/790"

    filename = system_tests.path(
        "$data_path/issue_712_poc.tif"
    )
    commands = ["$exiv2 -b -u -k pr $filename"]
    stdout = [""]
    stderr = [
        """$exiv2_exception_message """ + filename + """:
$filename: $kerFileContainsUnknownImageType
"""]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests


class SigmaLenses(metaclass=system_tests.CaseMeta):

    files = [
        "Sigma_120-300_DG_OS_HSM_Sport_lens.exv",
        "Sigma_20mm_F1.4_DG_HSM_A.exv",
        "Sigma_50mm_F1.4_DG_HSM_A.exv"
    ]

    commands = list(
        map(lambda fname: "$exiv2 -pa --grep lens/i $data_path/" + fname, files)
    )

    retval = 3 * [0]
    stderr = 3 * [""]
    stdout = [
        """Exif.Nikon3.LensType                         Byte        1  D G VR
Exif.Nikon3.Lens                             Rational    4  120-300mm F2.8
Exif.Nikon3.LensFStops                       Undefined   4  6
Exif.NikonLd3.LensIDNumber                   Byte        1  Sigma 120-300mm F2.8 DG OS HSM | S
Exif.NikonLd3.LensFStops                     Byte        1  F6.0
""",
        """Exif.Nikon3.LensType                         Byte        1  D G 
Exif.Nikon3.Lens                             Rational    4  20mm F1.4
Exif.Nikon3.LensFStops                       Undefined   4  7
Exif.NikonLd3.LensIDNumber                   Byte        1  Sigma 20mm F1.4 DG HSM | A
Exif.NikonLd3.LensFStops                     Byte        1  F7.0
""",
        """Exif.Nikon3.LensType                         Byte        1  D G 
Exif.Nikon3.Lens                             Rational    4  50mm F1.4
Exif.Nikon3.LensFStops                       Undefined   4  7
Exif.NikonLd3.LensIDNumber                   Byte        1  Sigma 50mm F1.4 DG HSM | A
Exif.NikonLd3.LensFStops                     Byte        1  F7.0
"""
    ]



# -*- coding: utf-8 -*-

import system_tests

class CheckLensZoomNikkor(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1026"

    filename = system_tests.path("$data_path/exiv2-bug1026.jpg")
    commands = [ "$exiv2 -q -pa -g Lens $filename" ]

    stdout = [ """Exif.Nikon3.LensType                         Byte        1  D G VR
Exif.Nikon3.Lens                             Rational    4  18-200mm F3.5-5.6
Exif.Nikon3.LensFStops                       Undefined   4  5.33333
Exif.NikonLd3.LensIDNumber                   Byte        1  Nikon AF-S DX VR Zoom-Nikkor 18-200mm f/3.5-5.6G IF-ED II
Exif.NikonLd3.LensFStops                     Byte        1  F5.3
"""
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests


class PentaxImageWithIssues(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1305"

    filename = "$data_path/IMGP0006-min.jpg"

    commands = ["$exiv2 $filename"]

    stdout = ["""File name       : $filename
File size       : 12341 Bytes
MIME type       : image/jpeg
Image size      : 0 x 0
Thumbnail       : None
Camera make     : PENTAX000000000000000000000000000000000000000000
Camera model    : 
Image timestamp : 
File number     : 
Exposure time   : 
Aperture        : 
Exposure bias   : 
Flash           : 
Flash bias      : 
Focal length    : 
Subject distance: 
ISO speed       : 
Exposure mode   : 
Metering mode   : 
Macro mode      : 
Image quality   : 
White balance   : 
Copyright       : 
Exif comment    : 

"""
    ]

    stderr = [""] * len(commands)
    retval = [0]  * len(commands)

    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

import system_tests


class DivByZeroInPrintIFD(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/262"

    filename = system_tests.path(
        "$data_path/7-printIFD-divbyzero-1"
    )
    commands = ["$exiv2 -pX $filename"]
    stdout = [""]
    stderr = [
        """$exiv2_exception_message $filename:
$filename: $kerFileContainsUnknownImageType
"""
    ]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests
import unittest

# test needs system_tests.BT.vv['enable_bmff']=1
bSkip=system_tests.BT.verbose_version().get('enable_bmff')!='1'
if bSkip:
    raise unittest.SkipTest('*** requires enable_bmff=1 ***')

class issue_1503_jxl_bmff(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/issue/1503"
    filename = "$data_path/Reagan.jxl"
    commands = ["$exiv2  -pS       $filename"
               ,"$exiv2  -g Date/i $filename"
               ,"$exiv2            $filename"
               ]
    retval = [ 0  ] * len(commands)
    stderr = [ "" ] * len(commands)
    stdin  = [ "" ] * len(commands)
    stdout = ["""Exiv2::BmffImage::boxHandler: JXL         0->12 
Exiv2::BmffImage::boxHandler: ftyp       12->20 brand: jxl 
Exiv2::BmffImage::boxHandler: Exif       32->5722 
Exiv2::BmffImage::boxHandler: xml      5754->5306 
Exiv2::BmffImage::boxHandler: jbrd    11060->1707 
Exiv2::BmffImage::boxHandler: jxlc    12767->20125 
""","""Exif.Image.DateTime                          Ascii      20  2016:09:13 11:58:16
Exif.Photo.DateTimeOriginal                  Ascii      20  2004:06:21 23:37:53
Exif.Photo.DateTimeDigitized                 Ascii      20  2004:06:21 23:37:53
Xmp.xmp.ModifyDate                           XmpText    25  2016-09-13T11:58:16+01:00
Xmp.xmp.CreateDate                           XmpText    25  2004-06-21T23:37:53+01:00
Xmp.xmp.MetadataDate                         XmpText    25  2016-09-13T11:58:16+01:00
Xmp.photoshop.DateCreated                    XmpText    10  2004-06-21
""","""File name       : $filename
File size       : 32892 Bytes
MIME type       : image/generic
Image size      : 200 x 130
Thumbnail       : image/jpeg, 4196 Bytes
Camera make     : NIKON CORPORATION
Camera model    : NIKON D1X
Image timestamp : 2004:06:21 23:37:53
File number     : 
Exposure time   : 1/125 s
Aperture        : F5
Exposure bias   : +1/3 EV
Flash           : No flash
Flash bias      : 
Focal length    : 42.0 mm
Subject distance: 
ISO speed       : 
Exposure mode   : Manual
Metering mode   : Center weighted average
Macro mode      : 
Image quality   : 
White balance   : Manual
Copyright       : 
Exif comment    : 

"""
]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path, check_no_ASAN_UBSAN_errors


class OutOfMemoryInJp2ImageReadMetadata(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1812

    Due to a missing bounds check, this test triggers a 4GB memory
    allocation. So the test will fail with a std::bad_alloc exception
    if less than 4GB is available.  On Linux, you can use `ulimit -v
    4000000` to reduce the available memory to slightly less than 4GB.
    """
    url = "https://github.com/Exiv2/exiv2/issues/1812"

    filename = path("$data_path/issue_1812_poc.jp2")
    commands = ["$exiv2 $filename"]
    stdout = [""]
    stderr = ["""$exiv2_exception_message $filename:
$kerCorruptedMetadata
"""]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests

class CheckDivisionByZero(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1080"

    filename = system_tests.path("$data_path/exiv2-bug1080.jpg")
    commands = [ "$exiv2 -pa $filename" ]

    stdout = [ """Exif.Image.ExifTag                           Long        1  26
Exif.Photo.ExposureBiasValue                 SRational   1  0 EV
"""
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests


class TestCvePoC(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/75"

    filename = "$data_path/008-invalid-mem"
    commands = ["$exiv2 " + filename]
    stdout = [""]
    stderr = ["""$exiv2_exception_message """ + filename + """:
$kerCorruptedMetadata
"""]
    retval = [1]

    def compare_stderr(self, i, command, got_stderr, expected_stderr):
        """
        Only check that an exception is thrown for this file
        ignore all the warnings on stderr on purpose.
        """
        self.assertIn(expected_stderr, got_stderr)



# -*- coding: utf-8 -*-

import system_tests


class ShadowingError(metaclass=system_tests.CaseMeta):

    commands = ["$exiv2 -PE -b $data_path/IMGP0020.exv"]
    stdout = [""]
    stderr = [""]
    retval = [0]

    def compare_stdout(self, i, command, got_stdout, expected_stdout):
        """
        We only really care about the LensInfo line and that exiv2 does not
        crash, which the return value check also ensures.
        """
        self.assertIn(
            "Exif.PentaxDng.LensInfo                      Undefined  69  131 0 0 255 0 40 148 111 65 69 6 238 65 78 153 80 40 1 73 107 251 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0",
            got_stdout
        )



# -*- coding: utf-8 -*-

import system_tests
import unittest

# test needs system_tests.BT.vv['enable_bmff']=1
bSkip=system_tests.BT.verbose_version().get('enable_bmff')!='1'
if bSkip:
    raise unittest.SkipTest('*** requires enable_bmff=1 ***')

class pr_1475_avif_avif(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1475"
    filename = "$data_path/avif.avif"
    if bSkip:
        commands=[]
        retval=[]
        stdin=[]
        stderr=[]
        stdout=[]
        print("*** test skipped.  requires enable_bmff=1***")
    else:
        commands = ["$exiv2 -pa $filename"
                   ,"$exiv2 -pS $filename"
                   ,"$exiv2 -pX $filename"
                   ,"$exiv2 -pC $filename"
                   ]
        retval = [ 0  ] * len(commands)
        stderr = [ "" ] * len(commands)
        stdin  = [ "" ] * len(commands)
        stdout = ["""Exif.Image.ExifTag                           Long        1  26
Exif.Photo.DateTimeOriginal                  Ascii      20  2018:08:15 11:37:35
Exif.Photo.SubSecTimeOriginal                Ascii       3  59
""","""Exiv2::BmffImage::boxHandler: ftyp        0->32 brand: avif
Exiv2::BmffImage::boxHandler: meta       32->304 
  Exiv2::BmffImage::boxHandler: hdlr       44->33 
  Exiv2::BmffImage::boxHandler: pitm       77->14 
  Exiv2::BmffImage::boxHandler: iinf       91->56 
    Exiv2::BmffImage::boxHandler: infe      105->21 ID =   1 av01 
    Exiv2::BmffImage::boxHandler: infe      126->21 ID =   2 Exif  *** Exif ***
  Exiv2::BmffImage::boxHandler: iloc      147->44 
       155 |       14 |   ID |    1 |    408,218726
       169 |       14 |   ID |    2 | 219134,    82
  Exiv2::BmffImage::boxHandler: iprp      191->119 
    Exiv2::BmffImage::boxHandler: ipco      199->88 
      Exiv2::BmffImage::boxHandler: ispe      207->20 pixelWidth_, pixelHeight_ = 1920, 1080
      Exiv2::BmffImage::boxHandler: colr      227->19 
      Exiv2::BmffImage::boxHandler: av1C      246->25 
      Exiv2::BmffImage::boxHandler: pixi      271->16 
    Exiv2::BmffImage::boxHandler: ipma      287->23 
  Exiv2::BmffImage::boxHandler: iref      310->26 
Exiv2::BMFF Exif: ID = 2 from,length = 219134,82
Exiv2::BmffImage::boxHandler: free      336->64 
Exiv2::BmffImage::boxHandler: mdat      400->218816 
""","",""]

class pr_1475_exif_xmp_avif(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1475"
    filename = "$data_path/avif_exif_xmp.avif"
    if bSkip:
        commands=[]
        retval=[]
        stdin=[]
        stderr=[]
        stdout=[]
        print("*** test skipped.  requires enable_bmff=1***")
    else:
        commands = ["$exiv2 -g Image.Make -g Date -g Xm -g Expo -g Flash $filename"
                   ,"$exiv2 -pS $filename"
                   ,"$exiv2 -pX $filename"
                   ,"$exiv2 -pC $filename"
                   ]
        retval = [ 0  ] * len(commands)
        stderr = [ "" ] * len(commands)
        stdin  = [ "" ] * len(commands)
        stdout = ["""Exif.Image.DateTime                          Ascii      20  2021:02:13 21:19:50
Xmp.iptcExt.DigitalSourceType                XmpText    61  http://cv.iptc.org/newscodes/digitalsourcetype/digitalCapture
Xmp.iptcExt.LocationCreated                  XmpBag      0  
Xmp.iptcExt.LocationShown                    XmpBag      0  
Xmp.iptcExt.ArtworkOrObject                  XmpBag      0  
Xmp.iptcExt.RegistryId                       XmpBag      0  
Xmp.xmpMM.DocumentID                         XmpText    52  gimp:docid:gimp:8cf1b29a-9b12-4dec-9930-f2c608b298de
Xmp.xmpMM.InstanceID                         XmpText    44  xmp.iid:33e259b4-7237-4b2a-87bf-4609986012a8
Xmp.xmpMM.OriginalDocumentID                 XmpText    44  xmp.did:02dfa8e8-ce11-4de7-971d-566a5bba5edb
Xmp.xmpMM.History                            XmpText     0  type="Seq"
Xmp.xmpMM.History[1]                         XmpText     0  type="Struct"
Xmp.xmpMM.History[1]/stEvt:action            XmpText     5  saved
Xmp.xmpMM.History[1]/stEvt:changed           XmpText     9  /metadata
Xmp.xmpMM.History[1]/stEvt:instanceID        XmpText    44  xmp.iid:446f9de2-f964-4680-8461-ddf05bfcc53e
Xmp.xmpMM.History[1]/stEvt:softwareAgent     XmpText    19  GIMP 2.99.5 (Linux)
Xmp.xmpMM.History[1]/stEvt:when              XmpText    25  2021-02-13T21:19:04+01:00
Xmp.xmpMM.History[2]                         XmpText     0  type="Struct"
Xmp.xmpMM.History[2]/stEvt:action            XmpText     5  saved
Xmp.xmpMM.History[2]/stEvt:changed           XmpText     1  /
Xmp.xmpMM.History[2]/stEvt:instanceID        XmpText    44  xmp.iid:017dc808-2418-4f70-99f3-aa05905adb88
Xmp.xmpMM.History[2]/stEvt:softwareAgent     XmpText    19  GIMP 2.99.5 (Linux)
Xmp.xmpMM.History[2]/stEvt:when              XmpText    25  2021-02-13T21:20:14+01:00
Xmp.plus.ModelReleaseStatus                  XmpText    38  None
Xmp.plus.ImageSupplier                       XmpSeq      0  
Xmp.plus.ImageCreator                        XmpSeq      0  
Xmp.plus.CopyrightOwner                      XmpSeq      0  
Xmp.plus.Licensor                            XmpSeq      0  
Xmp.GIMP.API                                 XmpText     3  3.0
Xmp.GIMP.Platform                            XmpText     5  Linux
Xmp.GIMP.TimeStamp                           XmpText    16  1613247614397805
Xmp.GIMP.Version                             XmpText     6  2.99.5
Xmp.dc.Format                                XmpText    10  image/avif
Xmp.dc.creator                               XmpSeq      1  type="Seq" Developer
Xmp.dc.description                           LangAlt     1  lang="x-default" This is a testfile
Xmp.dc.rights                                LangAlt     1  lang="x-default" It is forbidden to use for evil purposes
Xmp.dc.title                                 LangAlt     1  lang="x-default" Personal photo
Xmp.photoshop.AuthorsPosition                XmpText    18  Computer Scientist
Xmp.xmp.CreatorTool                          XmpText     4  GIMP
Xmp.xmp.Rating                               XmpText     1  5
Xmp.xmpRights.Marked                         XmpText     4  True
""","""Exiv2::BmffImage::boxHandler: ftyp        0->32 brand: avif
Exiv2::BmffImage::boxHandler: meta       32->379 
  Exiv2::BmffImage::boxHandler: hdlr       44->40 
  Exiv2::BmffImage::boxHandler: pitm       84->14 
  Exiv2::BmffImage::boxHandler: iloc       98->58 
       106 |       14 |   ID |    1 |   5418,  5445
       120 |       14 |   ID |    2 |    419,   316
       134 |       14 |   ID |    3 |    735,  4683
  Exiv2::BmffImage::boxHandler: iinf      156->109 
    Exiv2::BmffImage::boxHandler: infe      170->26 ID =   1 av01Color 
    Exiv2::BmffImage::boxHandler: infe      196->25 ID =   2 ExifExif  *** Exif ***
    Exiv2::BmffImage::boxHandler: infe      221->44 ID =   3 mimeXMP  *** XMP ***
  Exiv2::BmffImage::boxHandler: iref      265->40 
  Exiv2::BmffImage::boxHandler: iprp      305->106 
    Exiv2::BmffImage::boxHandler: ipco      313->75 
      Exiv2::BmffImage::boxHandler: ispe      321->20 pixelWidth_, pixelHeight_ = 120, 120
      Exiv2::BmffImage::boxHandler: pixi      341->16 
      Exiv2::BmffImage::boxHandler: av1C      357->12 
      Exiv2::BmffImage::boxHandler: colr      369->19 
    Exiv2::BmffImage::boxHandler: ipma      388->23 
Exiv2::BMFF Exif: ID = 2 from,length = 419,316
Exiv2::BMFF XMP: ID = 3 from,length = 735,4683
Exiv2::BmffImage::boxHandler: mdat      411->10452 
""","""<?xpacket begin="﻿" id="W5M0MpCehiHzreSzNTczkc9d"?>
<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="XMP Core 4.4.0-Exiv2">
 <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <rdf:Description rdf:about=""
    xmlns:iptcExt="http://iptc.org/std/Iptc4xmpExt/2008-02-29/"
    xmlns:xmpMM="http://ns.adobe.com/xap/1.0/mm/"
    xmlns:stEvt="http://ns.adobe.com/xap/1.0/sType/ResourceEvent#"
    xmlns:plus="http://ns.useplus.org/ldf/xmp/1.0/"
    xmlns:GIMP="http://www.gimp.org/xmp/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:photoshop="http://ns.adobe.com/photoshop/1.0/"
    xmlns:xmp="http://ns.adobe.com/xap/1.0/"
    xmlns:xmpRights="http://ns.adobe.com/xap/1.0/rights/"
   iptcExt:DigitalSourceType="http://cv.iptc.org/newscodes/digitalsourcetype/digitalCapture"
   xmpMM:DocumentID="gimp:docid:gimp:8cf1b29a-9b12-4dec-9930-f2c608b298de"
   xmpMM:InstanceID="xmp.iid:33e259b4-7237-4b2a-87bf-4609986012a8"
   xmpMM:OriginalDocumentID="xmp.did:02dfa8e8-ce11-4de7-971d-566a5bba5edb"
   plus:ModelReleaseStatus="http://ns.useplus.org/ldf/vocab/MR-NON"
   GIMP:API="3.0"
   GIMP:Platform="Linux"
   GIMP:TimeStamp="1613247614397805"
   GIMP:Version="2.99.5"
   dc:Format="image/avif"
   photoshop:AuthorsPosition="Computer Scientist"
   xmp:CreatorTool="GIMP"
   xmp:Rating="5"
   xmpRights:Marked="True">
   <iptcExt:LocationCreated>
    <rdf:Bag/>
   </iptcExt:LocationCreated>
   <iptcExt:LocationShown>
    <rdf:Bag/>
   </iptcExt:LocationShown>
   <iptcExt:ArtworkOrObject>
    <rdf:Bag/>
   </iptcExt:ArtworkOrObject>
   <iptcExt:RegistryId>
    <rdf:Bag/>
   </iptcExt:RegistryId>
   <xmpMM:History>
    <rdf:Seq>
     <rdf:li
      stEvt:action="saved"
      stEvt:changed="/metadata"
      stEvt:instanceID="xmp.iid:446f9de2-f964-4680-8461-ddf05bfcc53e"
      stEvt:softwareAgent="GIMP 2.99.5 (Linux)"
      stEvt:when="2021-02-13T21:19:04+01:00"/>
     <rdf:li
      stEvt:action="saved"
      stEvt:changed="/"
      stEvt:instanceID="xmp.iid:017dc808-2418-4f70-99f3-aa05905adb88"
      stEvt:softwareAgent="GIMP 2.99.5 (Linux)"
      stEvt:when="2021-02-13T21:20:14+01:00"/>
    </rdf:Seq>
   </xmpMM:History>
   <plus:ImageSupplier>
    <rdf:Seq/>
   </plus:ImageSupplier>
   <plus:ImageCreator>
    <rdf:Seq/>
   </plus:ImageCreator>
   <plus:CopyrightOwner>
    <rdf:Seq/>
   </plus:CopyrightOwner>
   <plus:Licensor>
    <rdf:Seq/>
   </plus:Licensor>
   <dc:creator>
    <rdf:Seq>
     <rdf:li>type="Seq" Developer</rdf:li>
    </rdf:Seq>
   </dc:creator>
   <dc:description>
    <rdf:Alt>
     <rdf:li xml:lang="x-default">This is a testfile</rdf:li>
    </rdf:Alt>
   </dc:description>
   <dc:rights>
    <rdf:Alt>
     <rdf:li xml:lang="x-default">It is forbidden to use for evil purposes</rdf:li>
    </rdf:Alt>
   </dc:rights>
   <dc:title>
    <rdf:Alt>
     <rdf:li xml:lang="x-default">Personal photo</rdf:li>
    </rdf:Alt>
   </dc:title>
  </rdf:Description>
 </rdf:RDF>
</x:xmpmeta>
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                           
<?xpacket end="w"?>""",""]

class pr_1475_metadata2_avif(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1475"
    filename = "$data_path/avif_metadata2.avif"
    if bSkip:
        commands=[]
        retval=[]
        stdin=[]
        stderr=[]
        stdout=[]
        print("*** test skipped.  requires enable_bmff=1***")
    else:
        commands = ["$exiv2 -g Image.Make -g Date -g Xm -g Expo -g Flash $filename"
                   ,"$exiv2 -pS $filename"
                   ,"$exiv2 -pX $filename"
                   ,"$exiv2 -pC $filename"
                   ]
        retval = [ 0  ] * len(commands)
        stderr = [ "" ] * len(commands)
        stdin  = [ "" ] * len(commands)
        stdout = ["""Exif.Image.DateTime                          Ascii      20  2021:02:13 21:25:32
Xmp.iptcExt.DigitalSourceType                XmpText    61  http://cv.iptc.org/newscodes/digitalsourcetype/digitalCapture
Xmp.iptcExt.LocationCreated                  XmpBag      0  
Xmp.iptcExt.LocationShown                    XmpBag      0  
Xmp.iptcExt.ArtworkOrObject                  XmpBag      0  
Xmp.iptcExt.RegistryId                       XmpBag      0  
Xmp.xmpMM.DocumentID                         XmpText    52  gimp:docid:gimp:e38231ee-2d28-42e6-b17e-6fa80c9fd85b
Xmp.xmpMM.InstanceID                         XmpText    44  xmp.iid:34774f89-42e3-4411-a691-f8cdb5b11def
Xmp.xmpMM.OriginalDocumentID                 XmpText    44  xmp.did:e2129b55-045f-4085-bbd2-e11d11edfa0c
Xmp.xmpMM.History                            XmpText     0  type="Seq"
Xmp.xmpMM.History[1]                         XmpText     0  type="Struct"
Xmp.xmpMM.History[1]/stEvt:action            XmpText     5  saved
Xmp.xmpMM.History[1]/stEvt:changed           XmpText     9  /metadata
Xmp.xmpMM.History[1]/stEvt:instanceID        XmpText    44  xmp.iid:928b36e2-0239-4616-847a-eff7a31816e3
Xmp.xmpMM.History[1]/stEvt:softwareAgent     XmpText    19  GIMP 2.99.5 (Linux)
Xmp.xmpMM.History[1]/stEvt:when              XmpText    25  2021-02-13T21:24:26+01:00
Xmp.xmpMM.History[2]                         XmpText     0  type="Struct"
Xmp.xmpMM.History[2]/stEvt:action            XmpText     5  saved
Xmp.xmpMM.History[2]/stEvt:changed           XmpText     1  /
Xmp.xmpMM.History[2]/stEvt:instanceID        XmpText    44  xmp.iid:23f267c5-c40a-45c8-947d-11313b4881c1
Xmp.xmpMM.History[2]/stEvt:softwareAgent     XmpText    19  GIMP 2.99.5 (Linux)
Xmp.xmpMM.History[2]/stEvt:when              XmpText    25  2021-02-13T21:25:41+01:00
Xmp.plus.ModelReleaseStatus                  XmpText    38  None
Xmp.plus.ImageSupplier                       XmpSeq      0  
Xmp.plus.ImageCreator                        XmpSeq      0  
Xmp.plus.CopyrightOwner                      XmpSeq      0  
Xmp.plus.Licensor                            XmpSeq      0  
Xmp.GIMP.API                                 XmpText     3  3.0
Xmp.GIMP.Platform                            XmpText     5  Linux
Xmp.GIMP.TimeStamp                           XmpText    16  1613247941462908
Xmp.GIMP.Version                             XmpText     6  2.99.5
Xmp.dc.Format                                XmpText    10  image/avif
Xmp.dc.creator                               XmpSeq      1  type="Seq" me
Xmp.dc.description                           LangAlt     1  lang="x-default" bla bla bla
Xmp.xmp.CreatorTool                          XmpText     4  GIMP
""","""Exiv2::BmffImage::boxHandler: ftyp        0->24 brand: avif
Exiv2::BmffImage::boxHandler: meta       24->356 
  Exiv2::BmffImage::boxHandler: hdlr       36->33 
  Exiv2::BmffImage::boxHandler: pitm       69->14 
  Exiv2::BmffImage::boxHandler: iloc       83->70 
        91 |       18 |   ID |    1 |    388,  4730
       109 |       18 |   ID |    2 |   5118,   412
       127 |       18 |   ID |    3 |   5530,  4254
  Exiv2::BmffImage::boxHandler: iinf      153->98 
    Exiv2::BmffImage::boxHandler: infe      167->21 ID =   1 av01 
    Exiv2::BmffImage::boxHandler: infe      188->21 ID =   2 Exif  *** Exif ***
    Exiv2::BmffImage::boxHandler: infe      209->42 ID =   3 mime  *** XMP ***
  Exiv2::BmffImage::boxHandler: iprp      251->89 
    Exiv2::BmffImage::boxHandler: ipco      259->59 
      Exiv2::BmffImage::boxHandler: colr      267->19 
      Exiv2::BmffImage::boxHandler: av1C      286->12 
      Exiv2::BmffImage::boxHandler: ispe      298->20 pixelWidth_, pixelHeight_ = 120, 120
    Exiv2::BmffImage::boxHandler: ipma      318->22 
  Exiv2::BmffImage::boxHandler: iref      340->40 
Exiv2::BMFF Exif: ID = 2 from,length = 5118,412
Exiv2::BMFF XMP: ID = 3 from,length = 5530,4254
Exiv2::BmffImage::boxHandler: mdat      380->9404 
""","""<?xpacket begin="﻿" id="W5M0MpCehiHzreSzNTczkc9d"?>
<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="XMP Core 4.4.0-Exiv2">
 <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <rdf:Description rdf:about=""
    xmlns:iptcExt="http://iptc.org/std/Iptc4xmpExt/2008-02-29/"
    xmlns:xmpMM="http://ns.adobe.com/xap/1.0/mm/"
    xmlns:stEvt="http://ns.adobe.com/xap/1.0/sType/ResourceEvent#"
    xmlns:plus="http://ns.useplus.org/ldf/xmp/1.0/"
    xmlns:GIMP="http://www.gimp.org/xmp/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:xmp="http://ns.adobe.com/xap/1.0/"
   iptcExt:DigitalSourceType="http://cv.iptc.org/newscodes/digitalsourcetype/digitalCapture"
   xmpMM:DocumentID="gimp:docid:gimp:e38231ee-2d28-42e6-b17e-6fa80c9fd85b"
   xmpMM:InstanceID="xmp.iid:34774f89-42e3-4411-a691-f8cdb5b11def"
   xmpMM:OriginalDocumentID="xmp.did:e2129b55-045f-4085-bbd2-e11d11edfa0c"
   plus:ModelReleaseStatus="http://ns.useplus.org/ldf/vocab/MR-NON"
   GIMP:API="3.0"
   GIMP:Platform="Linux"
   GIMP:TimeStamp="1613247941462908"
   GIMP:Version="2.99.5"
   dc:Format="image/avif"
   xmp:CreatorTool="GIMP">
   <iptcExt:LocationCreated>
    <rdf:Bag/>
   </iptcExt:LocationCreated>
   <iptcExt:LocationShown>
    <rdf:Bag/>
   </iptcExt:LocationShown>
   <iptcExt:ArtworkOrObject>
    <rdf:Bag/>
   </iptcExt:ArtworkOrObject>
   <iptcExt:RegistryId>
    <rdf:Bag/>
   </iptcExt:RegistryId>
   <xmpMM:History>
    <rdf:Seq>
     <rdf:li
      stEvt:action="saved"
      stEvt:changed="/metadata"
      stEvt:instanceID="xmp.iid:928b36e2-0239-4616-847a-eff7a31816e3"
      stEvt:softwareAgent="GIMP 2.99.5 (Linux)"
      stEvt:when="2021-02-13T21:24:26+01:00"/>
     <rdf:li
      stEvt:action="saved"
      stEvt:changed="/"
      stEvt:instanceID="xmp.iid:23f267c5-c40a-45c8-947d-11313b4881c1"
      stEvt:softwareAgent="GIMP 2.99.5 (Linux)"
      stEvt:when="2021-02-13T21:25:41+01:00"/>
    </rdf:Seq>
   </xmpMM:History>
   <plus:ImageSupplier>
    <rdf:Seq/>
   </plus:ImageSupplier>
   <plus:ImageCreator>
    <rdf:Seq/>
   </plus:ImageCreator>
   <plus:CopyrightOwner>
    <rdf:Seq/>
   </plus:CopyrightOwner>
   <plus:Licensor>
    <rdf:Seq/>
   </plus:Licensor>
   <dc:creator>
    <rdf:Seq>
     <rdf:li>type="Seq" me</rdf:li>
    </rdf:Seq>
   </dc:creator>
   <dc:description>
    <rdf:Alt>
     <rdf:li xml:lang="x-default">bla bla bla</rdf:li>
    </rdf:Alt>
   </dc:description>
  </rdf:Description>
 </rdf:RDF>
</x:xmpmeta>
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                           
<?xpacket end="w"?>""",""]




# -*- coding: utf-8 -*-

import system_tests


class TestCvePoC(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/51"

    filename = "$data_path/POC4"
    commands = ["$exiv2 $filename"]
    stdout = [
        """File name       : $filename
File size       : 35 Bytes
MIME type       : image/x-panasonic-rw2
Image size      : 0 x 0
"""
    ]
    stderr = [
        """Warning: Directory PanasonicRaw has an unexpected next pointer; ignored.
Error: Directory PanasonicRaw, entry 0x002e has invalid size 4294967295*1; skipping entry.
$filename: No Exif data found in the file
"""
    ]
    retval = ["$no_exif_data_found_retval"]



# -*- coding: utf-8 -*-

import system_tests


class TestCvePoC(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/208"

    filename = "$data_path/2018-01-09-exiv2-crash-001.tiff"
    commands = ["$exiv2 " + filename]
    retval = [1]
    stdout = [""]
    stderr = [
        """$exiv2_exception_message """ + filename + """:
$filename: $kerFileContainsUnknownImageType
"""]



# -*- coding: utf-8 -*-

import system_tests

class CheckSonyMinoltaLenses(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1145"

    filenames = ["$data_path/exiv2-bug1145a.exv",
                 "$data_path/exiv2-bug1145b.exv",
                 "$data_path/exiv2-bug1145c.exv",
                 "$data_path/exiv2-bug1145d.exv",
                 "$data_path/exiv2-bug1145e.exv",
    ]

    commands = [ "$exiv2 -pa -g Lens " + filenames[0],
                 "$exiv2 -pa -g Lens " + filenames[1],
                 "$exiv2 -pa -g Lens " + filenames[2],
                 "$exiv2 -pa -g Lens " + filenames[3],
                 "$exiv2 -pa -g Lens " + filenames[4],
    ]

    stdout = [ """Exif.Sony1.LensID                            Long        1  Tamron SP AF 11-18mm F4.5-5.6 Di II LD Aspherical IF
Exif.Photo.LensSpecification                 Rational    4  110/10 180/10 45/10 56/10
Exif.Photo.LensModel                         Ascii      20  DT 11-18mm F4.5-5.6
""",
              """Exif.Sony1.LensID                            Long        1  Tamron SP AF 17-50mm F2.8 XR Di II LD Aspherical
Exif.Photo.LensSpecification                 Rational    4  0/10 0/10 0/10 0/10
Exif.Photo.LensModel                         Ascii       5  ----
""",
              """Exif.Sony1.LensID                            Long        1  Tamron AF 18-200mm F3.5-6.3 XR Di II LD Aspherical [IF] Macro
Exif.Photo.LensSpecification                 Rational    4  0/10 0/10 0/10 0/10
Exif.Photo.LensModel                         Ascii       5  ----
""",
              """Exif.Sony1.LensID                            Long        1  Tamron SP 70-300mm F4-5.6 Di USD
Exif.Photo.LensSpecification                 Rational    4  700/10 3000/10 40/10 56/10
Exif.Photo.LensModel                         Ascii      16  70-300mm F4-5.6
""",
              """Exif.Sony1.LensID                            Long        1  Tamron SP AF 90mm F2.8 Di Macro
Exif.Photo.LensSpecification                 Rational    4  1000/10 1000/10 28/10 28/10
Exif.Photo.LensModel                         Ascii      17  100mm F2.8 Macro
""",
    ]
    stderr = [""] * len(commands)
    retval = [0] * len(commands)



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path

class MemoryLeakWebPImageDecodeChunks(metaclass=CaseMeta):
    """
    Test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1841
    """
    url = "https://github.com/Exiv2/exiv2/issues/1841"

    filename = path("$data_path/issue_1841_poc.webp")
    commands = ["$exiv2 $filename"]
    stdout = [""]
    stderr = ["""$exiv2_exception_message $filename:
This does not look like a TIFF image
"""]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path, check_no_ASAN_UBSAN_errors

class Jp2ImageEncodeJp2HeaderOutOfBoundsRead2(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/security/advisories/GHSA-v5g7-46xf-h728
    """
    url = "https://github.com/Exiv2/exiv2/security/advisories/GHSA-v5g7-46xf-h728"

    filename = path("$data_path/issue_ghsa_v5g7_46xf_h728_poc.exv")
    commands = ["$exiv2 $filename"]
    stdout = ["""File name       : $filename
File size       : 276 Bytes
MIME type       : application/rdf+xml
Image size      : 0 x 0
"""]
    stderr = ["""Error: XMP Toolkit error 201: Error in XMLValidator
Warning: Failed to decode XMP metadata.
$filename: No Exif data found in the file
"""]
    retval = [253]



import system_tests


class SegvInPngImageReadMetadata(
        metaclass=system_tests.CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/789
    """
    url = "https://github.com/Exiv2/exiv2/issues/789"

    filename = system_tests.path(
        "$data_path/issue_789_poc1.png"
    )
    commands = ["$exiv2 $filename"]
    stdout = [""]
    stderr = [""]
    retval = [1]

    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path

class Fix646_NikonAF22(metaclass=CaseMeta):

    url      =   "http://dev.exiv2.org/issues/646"
    filename =   "$data_path/_DSC8437.exv"
    commands = [
                  "$exiv2 -pa --grep NikonAf $filename"
]
    stdout   = [
        """Exif.NikonAf22.Version                       Undefined   4  1.01
Exif.NikonAf22.ContrastDetectAF              Byte        1  On
Exif.NikonAf22.AFAreaMode                    Byte        1  1
Exif.NikonAf22.PhaseDetectAF                 Byte        1  Off
Exif.NikonAf22.PrimaryAFPoint                Byte        1  0
Exif.NikonAf22.AFPointsUsed                  Byte        7  0 0 0 0 0 0 0
Exif.NikonAf22.AFImageWidth                  Short       1  8256
Exif.NikonAf22.AFImageHeight                 Short       1  5504
Exif.NikonAf22.AFAreaXPosition               Short       1  3950
Exif.NikonAf22.AFAreaYPosition               Short       1  2871
Exif.NikonAf22.AFAreaWidth                   Short       1  435
Exif.NikonAf22.AFAreaHeight                  Short       1  360
Exif.NikonAf22.ContrastDetectAFInFocus       Byte        2  1 0
"""
    ]
    stderr = [ "" ] * len(commands)
    retval = [ 0  ] * len(commands)



# -*- coding: utf-8 -*-

import system_tests


class CheckShutterInPentax(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1223"
    filenames = ["$data_path/RAW_PENTAX_K30.exv",
                 "$data_path/RAW_PENTAX_K100.exv"]

    commands = [
        "$exiv2 -pa --grep Shutter " + filenames[0],
        "$exiv2 -pa --grep Shutter " + filenames[1],
    ]

    stdout = [
        "Exif.PentaxDng.ShutterCount                  Undefined   4  115\n",
        "Exif.Pentax.ShutterCount                     Undefined   4  24757\n",
    ]

    stderr = [""] * len(stdout)
    retval = [0] * len(stdout)



# -*- coding: utf-8 -*-

import system_tests

class CheckNegativeValuesOfSByte(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1114"

    filename = system_tests.path("$data_path/exiv2-bug1114.jpg")
    commands = [ "$exiv2 -pv -g TuneAdj $filename" ]

    stdout = [ """0x0002 NikonAFT     AFFineTuneAdj               SByte       1  -2
"""
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests

class FujiFilm_IFD_Tags_pr1409(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1409"
    
    filename = "$data_path/exiv2-pr1409.exv"
    commands  = ["$exiv2 -g Fujifilm $filename"]
    stderr = [""]
    stdout = ["""Exif.Fujifilm.Version                        Undefined   4  48 49 51 48
Exif.Fujifilm.SerialNumber                   Ascii      48  FF02B4550110     593534373734180817925330110168
Exif.Fujifilm.Quality                        Ascii       8  NORMAL 
Exif.Fujifilm.Sharpness                      Short       1  Normal
Exif.Fujifilm.WhiteBalance                   Short       1  Auto
Exif.Fujifilm.Color                          Short       1  Normal
Exif.Fujifilm.FlashMode                      Short       1  (38976)
Exif.Fujifilm.FlashStrength                  SRational   1  0/100
Exif.Fujifilm.FocusMode                      Short       1  Auto
Exif.Fujifilm.SlowSync                       Short       1  Off
Exif.Fujifilm.PictureMode                    Short       1  Aperture-priority AE
Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.CropMode                       Short       1  None
Exif.Fujifilm.Continuous                     Short       1  Off
Exif.Fujifilm.SequenceNumber                 Short       1  0
Exif.Fujifilm.BlurWarning                    Short       1  Off
Exif.Fujifilm.FocusWarning                   Short       1  Off
Exif.Fujifilm.ExposureWarning                Short       1  Off
Exif.Fujifilm.DynamicRange                   Short       1  Standard
Exif.Fujifilm.FilmMode                       Short       1  F0/Standard (Provia)
Exif.Fujifilm.DynamicRangeSetting            Short       1  Raw
Exif.Fujifilm.DevelopmentDynamicRange        Short       1  100
Exif.Fujifilm.MinFocalLength                 Rational    1  1800/100
Exif.Fujifilm.MaxFocalLength                 Rational    1  5500/100
Exif.Fujifilm.MaxApertureAtMinFocal          Rational    1  280/100
Exif.Fujifilm.MaxApertureAtMaxFocal          Rational    1  400/100
Exif.Fujifilm.Rating                         Long        1  0
Exif.Fujifilm.RawImageFullWidth              Long        1  6384
Exif.Fujifilm.RawImageFullHeight             Long        1  4182
Exif.Fujifilm.BitsPerSample                  Long        1  14
Exif.Fujifilm.StripOffsets                   Long        1  2048
Exif.Fujifilm.StripByteCounts                Long        1  53395776
Exif.Fujifilm.BlackLevel                     Long       36  1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022 1022
Exif.Fujifilm.GeometricDistortionParams      SRational  19  3749/9 502/1420 710/1420 870/1420 1004/1420 1123/1420 1230/1420 1328/1420 1420/1420 1506/1420 35789/65536 64000/65536 92105/65536 121578/65536 152315/65536 183578/65536 216947/65536 251473/65536 286000/65536
Exif.Fujifilm.WB_GRBLevelsStandard           Long        8  302 371 826 17 302 626 485 21
Exif.Fujifilm.WB_GRBLevelsAuto               Long        3  302 561 552
Exif.Fujifilm.WB_GRBLevels                   Long        3  302 561 552
Exif.Fujifilm.ChromaticAberrationParams      SRational  29  3749/9 502/1420 710/1420 870/1420 1004/1420 1123/1420 1230/1420 1328/1420 1420/1420 1506/1420 4/65536 2/65536 -2/65536 -6/65536 -8/65536 -12/65536 -14/65536 -18/65536 -18/65536 -58/65536 -54/65536 -50/65536 -46/65536 -40/65536 -32/65536 -24/65536 -16/65536 -16/65536 3749/9
Exif.Fujifilm.VignettingParams               SRational  19  3749/9 502/1420 710/1420 870/1420 1004/1420 1123/1420 1230/1420 1328/1420 1420/1420 1506/1420 192752/2048 183960/2048 176751/2048 171126/2048 158275/2048 138976/2048 119700/2048 101213/2048 87673/2048
"""]
    retval = [0]



# -*- coding: utf-8 -*-

import itertools
import os.path

from system_tests import CaseMeta, path


def read_file(filename):
    with open(filename, 'r') as f:
        return f.read()


class AddMinusPSOption(metaclass=CaseMeta):

    cls_location = os.path.dirname(__file__)

    url = "http://dev.exiv2.org/issues/922"

    bug_jpg_file = path("$data_path/exiv2-bug922.jpg")
    IPTC_file = path("$data_path/iptc-psAPP13-wIPTCempty-psAPP13-wIPTC.jpg")
    files = [
        path("$data_path/{!s}".format(img))
        for img in "exiv2-bug922.png exiv2-bug922.tif exiv2-bug922a.jpg".split()
    ]

    png_bug_file = files[0]
    tif_bug_file = files[1]
    jpg_bug_file = files[2]

    commands = [
        "$exiv2 -pX $bug_jpg_file",
        "$exiv2 -pX $IPTC_file",
    ] + list(
        itertools.chain.from_iterable([
            "$exiv2 -pX " + fname,
            "$exiv2 -pS " + fname
        ] for fname in files)
    )

    stdout = [
        read_file(
            os.path.join(cls_location, "issue_922_exiv2_pX_bug922_jpg_output")
        ),
        """<?xpacket begin="\ufeff" id="W5M0MpCehiHzreSzNTczkc9d"?>
<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="XMP Core 4.1.1-Exiv2">
 <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <rdf:Description rdf:about=""
    xmlns:xap="http://ns.adobe.com/xap/1.0/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:photoshop="http://ns.adobe.com/photoshop/1.0/"
    xmlns:xapMM="http://ns.adobe.com/xap/1.0/mm/"
    xmlns:tiff="http://ns.adobe.com/tiff/1.0/"
    xmlns:exif="http://ns.adobe.com/exif/1.0/"
   xap:CreateDate="2009-06-03T17:02:22+02:00"
   xap:ModifyDate="2009-06-03T17:02:22+02:00"
   xap:MetadataDate="2009-06-03T17:02:22+02:00"
   xap:CreatorTool="Adobe Photoshop CS3 Macintosh"
   dc:format="image/jpeg"
   photoshop:ColorMode="3"
   photoshop:History=""
   xapMM:InstanceID="uuid:6E0DE597F251DE119368FCA8AE42D1CE"
   xapMM:DocumentID="uuid:6D0DE597F251DE119368FCA8AE42D1CE"
   tiff:Orientation="1"
   tiff:XResolution="720000/10000"
   tiff:YResolution="720000/10000"
   tiff:ResolutionUnit="2"
   tiff:NativeDigest="256,257,258,259,262,274,277,284,530,531,282,283,296,301,318,319,529,532,306,270,271,272,305,315,33432;A50D672D77BFA6B1794A470713D72F1B"
   exif:PixelXDimension="420"
   exif:PixelYDimension="300"
   exif:ColorSpace="-1"
   exif:NativeDigest="36864,40960,40961,37121,37122,40962,40963,37510,40964,36867,36868,33434,33437,34850,34852,34855,34856,37377,37378,37379,37380,37381,37382,37383,37384,37385,37386,37396,41483,41484,41486,41487,41488,41492,41493,41495,41728,41729,41730,41985,41986,41987,41988,41989,41990,41991,41992,41993,41994,41995,41996,42016,0,2,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,20,22,23,24,25,26,27,28,30;994BCC01E43CFBC321A79B1A803AAF9B">
   <xapMM:DerivedFrom rdf:parseType="Resource"/>
  </rdf:Description>
 </rdf:RDF>
</x:xmpmeta>
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                           
<?xpacket end="w"?>""",
        """<?xpacket begin="\ufeff" id="W5M0MpCehiHzreSzNTczkc9d"?>
<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="XMP Core 4.4.0-Exiv2">
 <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <rdf:Description rdf:about=""
    xmlns:dc="http://purl.org/dc/elements/1.1/">
   <dc:title>
    <rdf:Alt>
     <rdf:li xml:lang="x-default">this is the title</rdf:li>
    </rdf:Alt>
   </dc:title>
  </rdf:Description>
 </rdf:RDF>
</x:xmpmeta>
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                           
<?xpacket end="w"?>""",
        """STRUCTURE OF PNG FILE: $png_bug_file
 address | chunk |  length | data                           | checksum
       8 | IHDR  |      13 | ...@........                   | 0x7f775da4
      33 | zTXt  |    8769 | Raw profile type exif..x...[r. | 0x4a89d860
    8814 | zTXt  |     270 | Raw profile type iptc..x.=QKn. | 0x29f9e2d3
    9096 | iTXt  |    2524 | XML:com.adobe.xmp.....<?xpacke | 0x1df4a351
   11632 | iCCP  |    1404 | icc..x...i8........af\...w_3.. | 0x363e2409
   13048 | sBIT  |       3 | ...                            | 0xdbe14fe0
   13063 | zTXt  |      87 | Software..x...A.. ......B....} | 0xcd30861a
   13162 | IDAT  |    8192 | x...Y.$Wv&v.{.{l.T.......[w.=m | 0x1b2c97be
   21366 | IDAT  |    8192 | .4X.y.AR...4....:Ue..U.|1..:.. | 0xb3fecb3e
   29570 | IDAT  |    8192 | 'g.!... ...n...s..Jdz......... | 0xd040336d
   37774 | IDAT  |    8192 | ........k....CY/75I..u;.. .z.. | 0xf42a2c7d
   45978 | IDAT  |    8192 | .f>..]....UKqD2s.(.q....=x.l.\ | 0xc177fe83
   54182 | IDAT  |    8192 | .i.{!!B0...C!4.p..`D g`....... | 0x0e276268
   62386 | IDAT  |    8192 | .*.].4..Q..}(9...S0&.......T.9 | 0x297bb2db
   70590 | IDAT  |    8192 | ..k...6....g.1..}.].&.H....... | 0x05f6f4ef
   78794 | IDAT  |    8192 | .j..S.........z..!U.G0*.m%..09 | 0xe0946eb5
   86998 | IDAT  |    8192 | .....t.>!.....6^.<..;..?.$I..M | 0x843ecce0
   95202 | IDAT  |    8192 | W.&5.5J........FW`....3.N.9Pk; | 0x3a3dfeee
  103406 | IDAT  |    8192 | .....d.z".`...v=g-..-.c8...Z.5 | 0x65d6df49
  111610 | IDAT  |    8192 | .."...o<&."....1M....1&. ..5.. | 0x700b8cde
  119814 | IDAT  |    8192 | k........!..B*.....\*.(!..0.s. | 0x9b33b5b7
  128018 | IDAT  |    3346 | .Y.L@I$M.Z[.0A ...K#.t.0+.G(.j | 0x18044b20
  131376 | IEND  |       0 |                                | 0xae426082
""",
        """<?xpacket begin="\ufeff" id="W5M0MpCehiHzreSzNTczkc9d"?>
<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="XMP Core 4.4.0-Exiv2">
 <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <rdf:Description rdf:about=""
    xmlns:dc="http://purl.org/dc/elements/1.1/">
   <dc:title>
    <rdf:Alt>
     <rdf:li xml:lang="x-default">this is a title</rdf:li>
    </rdf:Alt>
   </dc:title>
  </rdf:Description>
 </rdf:RDF>
</x:xmpmeta>
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                           
<?xpacket end="w"?>""",
        """STRUCTURE OF TIFF FILE (MM): $tif_bug_file
 address |    tag                              |      type |    count |    offset | value
      10 | 0x0100 ImageWidth                   |     SHORT |        1 |           | 40
      22 | 0x0101 ImageLength                  |     SHORT |        1 |           | 470
      34 | 0x0102 BitsPerSample                |     SHORT |        3 |       182 | 8 8 8
      46 | 0x0103 Compression                  |     SHORT |        1 |           | 5
      58 | 0x0106 PhotometricInterpretation    |     SHORT |        1 |           | 2
      70 | 0x0111 StripOffsets                 |      LONG |        1 |           | 2694
      82 | 0x0112 Orientation                  |     SHORT |        1 |           | 6
      94 | 0x0115 SamplesPerPixel              |     SHORT |        1 |           | 3
     106 | 0x0116 RowsPerStrip                 |     SHORT |        1 |           | 1092
     118 | 0x0117 StripByteCounts              |      LONG |        1 |           | 5086
     130 | 0x011c PlanarConfiguration          |     SHORT |        1 |           | 1
     142 | 0x013d Predictor                    |     SHORT |        1 |           | 2
     154 | 0x0153 SampleFormat                 |     SHORT |        3 |       188 | 1 1 1
     166 | 0x02bc XMLPacket                    |      BYTE |     2500 |       194 | <?xpacket begin="..." id="W5M0Mp ...
END $tif_bug_file
""",
        """<?xpacket begin="\ufeff" id="W5M0MpCehiHzreSzNTczkc9d"?> <x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="XMP Core 4.4.0-Exiv2"> <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"> <rdf:Description rdf:about="" xmlns:xmp="http://ns.adobe.com/xap/1.0/" xmlns:dc="http://purl.org/dc/elements/1.1/" xmp:Rating="0" xmp:ModifyDate="2015-02-13T20:46:51-06:00"> <dc:title> <rdf:Alt> <rdf:li xml:lang="x-default">the rest drink soda</rdf:li> <rdf:li xml:lang="en-GB">the Brits are in the bar</rdf:li> <rdf:li xml:lang="de-DE">the Germans like beer</rdf:li> </rdf:Alt> </dc:title> </rdf:Description> </rdf:RDF> </x:xmpmeta>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 <?xpacket end="w"?>""",
        """STRUCTURE OF JPEG FILE: $jpg_bug_file
 address | marker       |  length | data
       0 | 0xffd8 SOI  
       2 | 0xffe1 APP1  |   14862 | Exif..II*......................
   14866 | 0xffe1 APP1  |    2720 | http://ns.adobe.com/xap/1.0/.<?x
   17588 | 0xffed APP13 |     110 | Photoshop 3.0.8BIM.......6.....
   17700 | 0xffe2 APP2  |    4094 | MPF.II*...............0100.....
   21796 | 0xffdb DQT   |     132 
   21930 | 0xffc0 SOF0  |      17 
   21949 | 0xffc4 DHT   |     418 
   22369 | 0xffda SOS  
""",
    ]
    stderr = [""] * 8
    retval = [0] * 8



import system_tests


class BufferOverReadInNikon1MakerNotePrint0x0088(
        metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/756"

    filename = system_tests.path(
        "$data_path/NikonMakerNotePrint0x088_overread"
    )
    commands = ["$exiv2 -pt --grep AFFocusPos $filename"]
    stdout = [
        """Exif.Nikon1.AFFocusPos                       Undefined   4  Invalid value; Center
"""
    ]
    stderr = [""]
    retval = [0]

    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

import system_tests


class AdditionOverflowInLoaderTiffGetData(metaclass=system_tests.CaseMeta):
    """
    Regression test for bug #366:
    https://github.com/Exiv2/exiv2/issues/366
    aka CVE-2018-12264
    https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2018-12264
    """
    filename = system_tests.path("$data_path/2-out-of-read-Poc")
    commands = ["$exiv2 -ep $filename"]
    stdout = [""]
    stderr = [
        """Warning: Directory Image, entry 0x0111: Strip 0 is outside of the data area; ignored.
$uncaught_exception $addition_overflow_message
"""
    ]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path, check_no_ASAN_UBSAN_errors

class Jp2ImageEncodeJp2HeaderOutOfBoundsRead2(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/security/advisories/GHSA-h9x9-4f77-336w
    """
    url = "https://github.com/Exiv2/exiv2/security/advisories/GHSA-h9x9-4f77-336w"

    filename = path("$data_path/issue_ghsa_h9x9_4f77_336w_poc.exv")
    commands = ["$exiv2 -P t $filename"]
    retval = [0]

    compare_stdout = check_no_ASAN_UBSAN_errors
    compare_stderr = check_no_ASAN_UBSAN_errors



#!/usr/bin/env python3 

# Utility for generating a fuzzing dictionary for Exiv2.
# See README.md (in this directory) for more information.

import sys
import json

# If `c` is not a printable ASCII character then escape it.
# For example, a zero byte is converted to '\x00'.
def escapeChar(c):
    if c == '\\':
        return '\\\\'
    elif c == '"':
        return '\\"'
    elif c.isascii() and c.isprintable():
        return c
    else:
        return '\\x{:02X}'.format(ord(c))

def escapeString(str):
    return ''.join(map(lambda c: escapeChar(chr(c)), bytes(str, 'utf-8')))

if len(sys.argv) < 2:
    print("usage: mkdict.py dict.json")
    sys.exit(1)

f = open(sys.argv[1], 'r')
dict_json = json.loads(f.read())
tuples = dict_json["#select"]["tuples"]
for r in tuples:
    print('"' + escapeString(r[0]) + '"')



# -*- coding: utf-8 -*-

import system_tests


class TestFirstPoC(metaclass=system_tests.CaseMeta):
    """
    Regression test for the first bug described in:
    https://github.com/Exiv2/exiv2/issues/246
    """
    url = "https://github.com/Exiv2/exiv2/issues/246"

    filename = "$data_path/1-string-format.jpg"
    commands = ["$exiv2 -pS " + filename]
    stdout = [
        """STRUCTURE OF JPEG FILE: """ + filename + """
 address | marker       |  length | data
       0 | 0xffd8 SOI  
       2 | 0xffe1 APP1  |      60 | Exif..II*.....0.i...........0000
"""]


    stderr = ["""$exiv2_exception_message """ + filename + """:
$kerNoImageInInputData
"""]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path
@CopyTmpFiles("$data_path/issue_ghsa_mvc4_g5pv_4qqq_poc.jpg")

class JpegBasePrintStructureInfiniteLoop(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/security/advisories/GHSA-mvc4-g5pv-4qqq
    """
    url = "https://github.com/Exiv2/exiv2/security/advisories/GHSA-mvc4-g5pv-4qqq"

    filename = path("$tmp_path/issue_ghsa_mvc4_g5pv_4qqq_poc.jpg")
    commands = ["$exiv2 -d I rm $filename"]
    stdout = [""]
    stderr = [
"""Exiv2 exception in erase action for file $filename:
$kerFailedToReadImageData
"""]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests
import itertools

class CheckSony6000WithoutLensModels(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1153"

    filenames = [
        "$data_path/exiv2-bug1153{E}{i}.exv".format(E=E, i=i)
        for E, i in itertools.product(
                ['A', 'J'],
                "a b c d e f g h i j k".split()
        )
    ]

    commands = [
        "$exiv2 -pa -g Lens {!s}".format(fname) for fname in filenames
    ]

    stdout = [
        """Exif.Sony2.LensID                            Long        1  Sony E 50mm F1.8 OSS
Exif.Photo.LensSpecification                 Rational    4  500/10 500/10 18/10 18/10
Exif.Photo.LensModel                         Ascii      16  E 50mm F1.8 OSS
""",
        """Exif.Sony2.LensID                            Long        1  Sony E 50mm F1.8 OSS
Exif.Photo.LensSpecification                 Rational    4  500/10 500/10 18/10 18/10
Exif.Photo.LensModel                         Ascii      16  E 50mm F1.8 OSS
""",
        """Exif.Sony2.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
Exif.Photo.LensSpecification                 Rational    4  160/10 500/10 35/10 56/10
Exif.Photo.LensModel                         Ascii      26  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony2.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
Exif.Photo.LensSpecification                 Rational    4  160/10 500/10 35/10 56/10
Exif.Photo.LensModel                         Ascii      26  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony2.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
Exif.Photo.LensSpecification                 Rational    4  160/10 500/10 35/10 56/10
Exif.Photo.LensModel                         Ascii      26  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony2.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
Exif.Photo.LensSpecification                 Rational    4  160/10 500/10 35/10 56/10
Exif.Photo.LensModel                         Ascii      26  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony2.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
Exif.Photo.LensSpecification                 Rational    4  160/10 500/10 35/10 56/10
Exif.Photo.LensModel                         Ascii      26  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony2.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
Exif.Photo.LensSpecification                 Rational    4  160/10 500/10 35/10 56/10
Exif.Photo.LensModel                         Ascii      26  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony2.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
Exif.Photo.LensSpecification                 Rational    4  160/10 500/10 35/10 56/10
Exif.Photo.LensModel                         Ascii      26  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony2.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
Exif.Photo.LensSpecification                 Rational    4  160/10 500/10 35/10 56/10
Exif.Photo.LensModel                         Ascii      26  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony2.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
Exif.Photo.LensSpecification                 Rational    4  160/10 500/10 35/10 56/10
Exif.Photo.LensModel                         Ascii      26  E PZ 16-50mm F3.5-5.6 OSS
""",

        """Exif.Sony1.LensID                            Long        1  Sony E 50mm F1.8 OSS
""",
        """Exif.Sony1.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony1.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony1.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony1.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony1.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony1.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony1.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony1.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony1.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
""",
        """Exif.Sony1.LensID                            Long        1  E PZ 16-50mm F3.5-5.6 OSS
""",
    ]
    stderr = [""] * len(commands)
    retval = [0] * len(commands)



# -*- coding: utf-8 -*-

import system_tests

class CanonSigmaLens_28mm_F14_DG_HSM_A(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1455"
    
    filename = "$data_path/Sigma_28mm_F1.4_DG_HSM_A_for_EOS.exv"
    commands = ["$exiv2 -pa --grep lensType/i $filename"]
    stderr = [""]
    stdout = [""
        """Exif.CanonCs.LensType                        Short       1  Sigma 28mm f/1.4 DG HSM | A
"""
]
    retval = [0]


class CanonSigmaLens_35mm_F14_DG_HSM_A(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1455"
    
    filename = "$data_path/Sigma_35mm_F1.4_DG_HSM_A_for_EOS.exv"
    commands = ["$exiv2 -pa --grep lensType/i $filename"]
    stderr = [""]
    stdout = [""
        """Exif.CanonCs.LensType                        Short       1  Sigma 35mm f/1.4 DG HSM | A
"""
]
    retval = [0]

class CanonSigmaLens_40mm_F14_DG_HSM_A(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1455"
    
    filename = "$data_path/Sigma_40mm_F1.4_DG_HSM_A_for_EOS.exv"
    commands = ["$exiv2 -pa --grep lensType/i $filename"]
    stderr = [""]
    stdout = [""
        """Exif.CanonCs.LensType                        Short       1  Sigma 40mm f/1.4 DG HSM | A
"""
]
    retval = [0]

class CanonSigmaLens_50mm_F14_DG_HSM_A(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1455"
    
    filename = "$data_path/Sigma_50mm_F1.4_DG_HSM_A_for_EOS.exv"
    commands = ["$exiv2 -pa --grep lensType/i $filename"]
    stderr = [""]
    stdout = [""
        """Exif.CanonCs.LensType                        Short       1  Sigma 50mm f/1.4 DG HSM | A
"""
]
    retval = [0]

class CanonSigmaLens_14_24mm_F28_DG_HSM_A(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1455"
    
    filename = "$data_path/Sigma_14-24mm_F2.8_DG_HSM_A_for_EOS.exv"
    commands = ["$exiv2 -pa --grep lensType/i $filename"]
    stderr = [""]
    stdout = [""
        """Exif.CanonCs.LensType                        Short       1  Sigma 14-24mm f/2.8 DG HSM | A
"""
]
    retval = [0]

class CanonSigmaLens_60_600mm_F45_63_DG_OS_HSM_S(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1455"
    
    filename = "$data_path/Sigma_60-600mm_F4.5-6.3_DG_OS_HSM_S_for_EOS.exv"
    commands = ["$exiv2 -pa --grep lensType/i $filename"]
    stderr = [""]
    stdout = [""
        """Exif.CanonCs.LensType                        Short       1  Sigma 60-600mm f/4.5-6.3 DG OS HSM | S
"""
]
    retval = [0]




# -*- coding: utf-8 -*-

import system_tests


class OutOfBoundsReadInPrintStructure(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/263"

    filenames = [
        system_tests.path("$data_path/9-printStructure-outbound-read-1"),
        system_tests.path("$data_path/10-printStructure-outbound-read-2")
    ]

    commands = ["$exiv2 -pR " + fname for fname in filenames]

    retval = [1] * 2

    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors
    compare_stdout = system_tests.check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

import system_tests


class IptcFieldParsingTooStrict(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/501"

    filename = "$data_path/exiv2-bug501.jpg"

    commands = ["$exiv2 -u -pi $filename"]

    encodings = ['windows-1252']

    stdout = ["""Iptc.Application2.RecordVersion              Short       1  2
Iptc.Application2.Caption                    String     15  Porträtt på Eva
Iptc.Application2.Writer                     String     10  Elsa Nordh
Iptc.Application2.Byline                     String     10  Elsa Nordh
Iptc.Application2.ObjectName                 String      7  Hundbus
Iptc.Application2.DateCreated                Date        8  2006-10-31
Iptc.Application2.City                       String      8  Göteborg
Iptc.Application2.ProvinceState              String     15  Västra Götaland
Iptc.Application2.CountryName                String      6  Sweden
Iptc.Application2.Keywords                   String      3  Eva
Iptc.Application2.Keywords                   String      7  Gallery
Iptc.Application2.Keywords                   String     21  Framkalla jul06 mamma
Iptc.Application2.Copyright                  String     19  http://nordharna.se
Iptc.Application2.ExpirationTime             String      2  SE
Iptc.Application2.SubLocation                String      9  KÃ¥lltorp
Iptc.Application2.0x00d1                     String      9  GÃ¶teborg
Iptc.Application2.0x00d2                     String     17  VÃ¤stra GÃ¶taland
Iptc.Application2.0x00d3                     String      6  416 81
Iptc.Application2.0x00d4                     String      6  Sweden
Iptc.Application2.0x00d5                     String     19  +46-(0)736-21 50 74
Iptc.Application2.0x00d6                     String     17  foto@nordharna.se
Iptc.Application2.0x00d7                     String     27  http://gallery.nordharna.se
Iptc.Application2.0x00e3                     String     27  http://gallery.nordharna.se
Iptc.Application2.0x00f2                     String      1  4
"""]
    stderr = ["""Warning: Unsupported time format
"""]
    retval = [0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path
@CopyTmpFiles("$data_path/test_issue_1472.jpg")

class test_issue_1472Test(metaclass=CaseMeta):

    filename  = path("$tmp_path/test_issue_1472.jpg")
    exvfile   = path("$tmp_path/test_issue_1472.exv")
    commands  = [ "rm     -rf                    $exvfile" 
                , "$exiv2 -pa --grep LensType2   $filename"
                , "$exiv2  ex                    $filename"
                , "$exiv2 -pa --grep LensType2   $exvfile"
                , "$exiv2  rm                    $filename"
                , "$exiv2 -pa --grep LensType2   $filename"
                , "$exiv2  in                    $filename"
                , "$exiv2 -pa --grep LensType2   $filename"
                ]
    stdout   =  [""
                ,"Exif.Sony2010e.LensType2                     Short       1  1024\n"
                ,""
                ,"Exif.Sony2010e.LensType2                     Short       1  1024\n"
                ,""
                ,""
                ,""
                ,"Exif.Sony2010e.LensType2                     Short       1  1024\n"
                ]
    stderr = [""]*len(commands)
    retval = [ 0,0,0,0,0,1,0,0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class InvalidDateXMP(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1713
    """
    url = "https://github.com/Exiv2/exiv2/issues/1713"

    filename = path("$data_path/issue_1713_poc.xmp")
    commands = ["$exiv2 -Ph $filename"]

    stderr = [
"""Error: XMP Toolkit error 201: Error in XMLValidator
Warning: Failed to decode XMP metadata.
"""
]
    retval = [0]

    def compare_stdout(self, i, command, got_stdout, expected_stdout):
        """ We don't care about the stdout, just don't crash """
        pass



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class RafNoTiffRegression(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1791
    """
    url = "https://github.com/Exiv2/exiv2/issues/1791"

    filename1 = path("$data_path/issue_1791_old.raf")
    filename2 = path("$data_path/issue_1791_new.raf")
    commands = ["$exiv2 -pa $filename1", "$exiv2 -pa $filename2"]
    stdout = ["""Exif.Image2.JPEGInterchangeFormat            Long        1  104
Exif.Image2.JPEGInterchangeFormatLength      Long        1  12
""",
              """Exif.Image2.JPEGInterchangeFormat            Long        1  104
Exif.Image2.JPEGInterchangeFormatLength      Long        1  12
Exif.Image.NewSubfileType                    Long        1  Primary image
"""]
    stderr = ["", ""]
    retval = [0, 0]



# -*- coding: utf-8 -*-

import system_tests


@system_tests.CopyFiles("$data_path/exiv2-empty.jpg")
class NonIntrusiveWriteUpdateIFD(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/452"

    filename = "$data_path/exiv2-empty_copy.jpg"
    commands = [
        """$exiv2 -u -v -M"set Exif.GPSInfo.GPSLatitude SRational -1/3 -2/3 -3/3" $filename""",
        "$exiv2 -u -pv $filename",
        """$exiv2 -u -v -M"set Exif.GPSInfo.GPSLatitude Rational 1/3 2/3 3/3" $filename""",
        "$exiv2 -u -pv $filename",

    ]
    stdout = [
        """File 1/1: $filename
Set Exif.GPSInfo.GPSLatitude "-1/3 -2/3 -3/3" (SRational)
""",
        """0x8825 Image        GPSTag                      Long        1  26
0x0002 GPSInfo      GPSLatitude                 SRational   3  -1/3 -2/3 -3/3
""",
        """File 1/1: $filename
Set Exif.GPSInfo.GPSLatitude "1/3 2/3 3/3" (Rational)
""",
        """0x8825 Image        GPSTag                      Long        1  26
0x0002 GPSInfo      GPSLatitude                 Rational    3  1/3 2/3 3/3
"""]
    stderr = [""] * 4
    retval = [0] * 4



# -*- coding: utf-8 -*-

import system_tests

# extended to also test include shutter mode and mechanical shutter count (pr 1444)
class Nikon_LensData8(metaclass=system_tests.CaseMeta):
    url      = "https://github.com/Exiv2/exiv2/pull/1437"
    filename = "$data_path/CH0_0174.exv"
    commands = ["$exiv2 -g lens/i -g aperture/i -g shutter/i $filename"]
    stderr   = [""]
    stdout   = ["""Exif.Nikon3.ShutterMode                      Short       1  Auto (Electronic Front Curtain)
Exif.Nikon3.MechanicalShutterCount           Long        1  174
Exif.Nikon3.LensType                         Byte        1  D G VR
Exif.Nikon3.Lens                             Rational    4  70-200mm F2.8
Exif.Nikon3.LensFStops                       Undefined   4  6
Exif.NikonLd4.AFAperture                     Byte        1  F2.8
Exif.NikonLd4.LensIDNumber                   Byte        1  Nikon AF-S VR Zoom-Nikkor 70-200mm f/2.8G IF-ED
Exif.NikonLd4.LensFStops                     Byte        1  F6.0
Exif.NikonLd4.MaxApertureAtMinFocal          Byte        1  F2.8
Exif.NikonLd4.MaxApertureAtMaxFocal          Byte        1  F2.8
Exif.NikonLd4.EffectiveMaxAperture           Byte        1  F2.8
Exif.NikonLd4.LensID                         Short       1  0
Exif.NikonLd4.MaxAperture                    Short       1  F0.5
Exif.Nikon3.ShutterCount                     Long        1  174
Exif.Photo.LensSpecification                 Rational    4  700/10 2000/10 280/100 280/100
Exif.Photo.LensMake                          Ascii       6  
Exif.Photo.LensModel                         Ascii      65  
Exif.Photo.LensSerialNumber                  Ascii      11  
"""]
    retval = [0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path, check_no_ASAN_UBSAN_errors

class EmptyStringXmpTextValueRead(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1819
    """
    url = "https://github.com/Exiv2/exiv2/issues/1819"

    filename = path("$data_path/issue_1819_poc.exv")
    commands = ["$exiv2 -q $filename"]
    stdout = ["""File name       : $filename
File size       : 1088 Bytes
MIME type       : application/rdf+xml
Image size      : 0 x 0
"""]
    stderr = ["""$filename: No Exif data found in the file
"""]
    retval = [253]



# -*- coding: utf-8 -*-

import system_tests
import os
import unittest

bSkip=not (os.path.isfile(system_tests.unit_tests) or os.path.isfile(system_tests.unit_tests + '.exe'))
if bSkip:
    msg='*** %s does not exist ***' % system_tests.unit_tests
    print(msg)
    raise unittest.SkipTest(msg)

system_tests.BT.runTest(system_tests.unit_tests,raw=True)
print('')



# -*- coding: utf-8 -*-

import system_tests


class NotValidIccProfile(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1247"
    filename = "$data_path/exiv2-bug1247.jpg"

    commands = [
        """$exiv2 -pa $filename"""
    ]

    stdout = [""]

    stderr = [ """Exiv2 exception in print action for file $filename:
$kerFailedToReadImageData
"""
    ]

    retval = [1]



# -*- coding: utf-8 -*-

import argparse
import functools
import os
import sys
import unittest
from fnmatch import fnmatchcase

import system_tests


class MyTestLoader(unittest.TestLoader):
    testNamePatterns = None

    def getTestCaseNames(self, testCaseClass):
        """
        Customize this code to allow you to filter test methods through testNamePatterns.
        """
        def shouldIncludeMethod(attrname):
            if not attrname.startswith(self.testMethodPrefix):
                return False
            testFunc = getattr(testCaseClass, attrname)
            if not callable(testFunc):
                return False
            return self.testNamePatterns is None or \
                 any(fnmatchcase(attrname, pattern) for pattern in self.testNamePatterns)

        testFnNames = list(filter(shouldIncludeMethod, dir(testCaseClass)))
        if self.sortTestMethodsUsing:
            testFnNames.sort(key=functools.cmp_to_key(self.sortTestMethodsUsing))
        return testFnNames


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description="The system test suite")
    parser.add_argument(
        "--config_file",
        type=str,
        nargs=1,
        help="Path to the suite's configuration file",
        default=['suite.conf']
    )
    parser.add_argument(
        "--verbose", "-v",
        action='count',
        help="verbosity level",
        default=1
    )
    parser.add_argument(
        "--debug",
        help="enable debugging output",
        action='store_true'
    )
    parser.add_argument(
        "dir_or_file",
        help="root directory under which the testsuite searches for tests or a"
        "single file which tests are run (defaults to the config file's"
        "location)",
        default=None,
        type=str,
        nargs='?'
    )

    args          = parser.parse_args()
    if 'VERBOSE' in os.environ:
        args.verbose = 2 
    conf_file     = args.config_file[0]
    DEFAULT_ROOT  = os.path.abspath(os.path.dirname(conf_file))
    system_tests.set_debug_mode(args.debug)
    system_tests.configure_suite(conf_file)

    testLoader = MyTestLoader()
    testLoader.testMethodPrefix = ''
    testLoader.testNamePatterns = ['test*', '*test']
    if args.dir_or_file is None or os.path.isdir(args.dir_or_file):
        discovered_tests = testLoader.discover(
            args.dir_or_file or DEFAULT_ROOT
        )
    elif os.path.isfile(args.dir_or_file):
        discovered_tests = testLoader.discover(
            os.path.dirname(args.dir_or_file),
            pattern=os.path.split(args.dir_or_file)[1],
        )
    else:
        print(
            "WARNING: Invalid search location, falling back to {!s}"
            .format(DEFAULT_ROOT),
            file=sys.stderr
        )
        discovered_tests = testLoader.discover(
            DEFAULT_ROOT
        )

    test_res = unittest.runner.TextTestRunner(verbosity=args.verbose)\
                              .run(discovered_tests)

    sys.exit(0 if len(test_res.failures) + len(test_res.errors) == 0 else 1)



# -*- coding: utf-8 -*-

import itertools
import system_tests

def char_range(c1, c2):
    """Generates the characters from `c1` to `c2`, inclusive."""
    for c in range(ord(c1), ord(c2)+1):
        yield chr(c)

ORIGINAL_FILES = ["$data_path/exiv2-bug1179" + case + ".exv"
    for case in char_range('a', 'j')]

def make_command(filename):
    return ["$exiv2 -pa --grep fuji/i " + filename]


class CheckFilmMode(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1179"

    commands = list(itertools.chain.from_iterable(
            make_command(fname) for fname in ORIGINAL_FILES))

    stdout = [
        """Exif.Fujifilm.Version                        Undefined   4  48 49 51 48
Exif.Fujifilm.SerialNumber                   Ascii      48  FFDT22794526     593332303134151113535030217060
Exif.Fujifilm.Quality                        Ascii       8  NORMAL 
Exif.Fujifilm.Sharpness                      Short       1  Normal
Exif.Fujifilm.WhiteBalance                   Short       1  Auto
Exif.Fujifilm.Color                          Short       1  Normal
Exif.Fujifilm.FlashMode                      Short       1  Off
Exif.Fujifilm.FlashStrength                  SRational   1  0/100
Exif.Fujifilm.Macro                          Short       1  Off
Exif.Fujifilm.FocusMode                      Short       1  Auto
Exif.Fujifilm.SlowSync                       Short       1  Off
Exif.Fujifilm.PictureMode                    Short       1  Program AE
Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.Continuous                     Short       1  Off
Exif.Fujifilm.SequenceNumber                 Short       1  0
Exif.Fujifilm.BlurWarning                    Short       1  Off
Exif.Fujifilm.FocusWarning                   Short       1  Off
Exif.Fujifilm.ExposureWarning                Short       1  Off
Exif.Fujifilm.DynamicRange                   Short       1  Standard
Exif.Fujifilm.FilmMode                       Short       1  F0/Standard (Provia)
Exif.Fujifilm.DynamicRangeSetting            Short       1  Auto (100-400%)
Exif.Fujifilm.Rating                         Long        1  0
""",
        """Exif.Fujifilm.Version                        Undefined   4  48 49 51 48
Exif.Fujifilm.SerialNumber                   Ascii      48  FFDT22794526     593332303134151113535030217060
Exif.Fujifilm.Quality                        Ascii       8  NORMAL 
Exif.Fujifilm.Sharpness                      Short       1  Normal
Exif.Fujifilm.WhiteBalance                   Short       1  Auto
Exif.Fujifilm.Color                          Short       1  Normal
Exif.Fujifilm.FlashMode                      Short       1  Off
Exif.Fujifilm.FlashStrength                  SRational   1  0/100
Exif.Fujifilm.Macro                          Short       1  Off
Exif.Fujifilm.FocusMode                      Short       1  Auto
Exif.Fujifilm.SlowSync                       Short       1  Off
Exif.Fujifilm.PictureMode                    Short       1  Program AE
Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.Continuous                     Short       1  Off
Exif.Fujifilm.SequenceNumber                 Short       1  0
Exif.Fujifilm.BlurWarning                    Short       1  Off
Exif.Fujifilm.FocusWarning                   Short       1  Off
Exif.Fujifilm.ExposureWarning                Short       1  Off
Exif.Fujifilm.DynamicRange                   Short       1  Standard
Exif.Fujifilm.FilmMode                       Short       1  F2/Fujichrome (Velvia)
Exif.Fujifilm.DynamicRangeSetting            Short       1  Auto (100-400%)
Exif.Fujifilm.Rating                         Long        1  0
""", #b
        """Exif.Fujifilm.Version                        Undefined   4  48 49 51 48
Exif.Fujifilm.SerialNumber                   Ascii      48  FFDT22794526     593332303134151113535030217060
Exif.Fujifilm.Quality                        Ascii       8  NORMAL 
Exif.Fujifilm.Sharpness                      Short       1  Normal
Exif.Fujifilm.WhiteBalance                   Short       1  Auto
Exif.Fujifilm.Color                          Short       1  Normal
Exif.Fujifilm.FlashMode                      Short       1  Off
Exif.Fujifilm.FlashStrength                  SRational   1  0/100
Exif.Fujifilm.Macro                          Short       1  Off
Exif.Fujifilm.FocusMode                      Short       1  Auto
Exif.Fujifilm.SlowSync                       Short       1  Off
Exif.Fujifilm.PictureMode                    Short       1  Program AE
Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.Continuous                     Short       1  Off
Exif.Fujifilm.SequenceNumber                 Short       1  0
Exif.Fujifilm.BlurWarning                    Short       1  Off
Exif.Fujifilm.FocusWarning                   Short       1  Off
Exif.Fujifilm.ExposureWarning                Short       1  Off
Exif.Fujifilm.DynamicRange                   Short       1  Standard
Exif.Fujifilm.FilmMode                       Short       1  F1b/Studio Portrait Smooth Skin Tone (Astia)
Exif.Fujifilm.DynamicRangeSetting            Short       1  Auto (100-400%)
Exif.Fujifilm.Rating                         Long        1  0
""",
    """Exif.Fujifilm.Version                        Undefined   4  48 49 51 48
Exif.Fujifilm.SerialNumber                   Ascii      48  FFDT22794526     593332303134151113535030217060
Exif.Fujifilm.Quality                        Ascii       8  NORMAL 
Exif.Fujifilm.Sharpness                      Short       1  Normal
Exif.Fujifilm.WhiteBalance                   Short       1  Auto
Exif.Fujifilm.Color                          Short       1  Normal
Exif.Fujifilm.FlashMode                      Short       1  Off
Exif.Fujifilm.FlashStrength                  SRational   1  0/100
Exif.Fujifilm.Macro                          Short       1  Off
Exif.Fujifilm.FocusMode                      Short       1  Auto
Exif.Fujifilm.SlowSync                       Short       1  Off
Exif.Fujifilm.PictureMode                    Short       1  Program AE
Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.Continuous                     Short       1  Off
Exif.Fujifilm.SequenceNumber                 Short       1  0
Exif.Fujifilm.BlurWarning                    Short       1  Off
Exif.Fujifilm.FocusWarning                   Short       1  Off
Exif.Fujifilm.ExposureWarning                Short       1  Off
Exif.Fujifilm.DynamicRange                   Short       1  Standard
Exif.Fujifilm.FilmMode                       Short       1  Classic Chrome
Exif.Fujifilm.DynamicRangeSetting            Short       1  Auto (100-400%)
Exif.Fujifilm.Rating                         Long        1  0
""",
    """Exif.Fujifilm.Version                        Undefined   4  48 49 51 48
Exif.Fujifilm.SerialNumber                   Ascii      48  FFDT22794526     593332303134151113535030217060
Exif.Fujifilm.Quality                        Ascii       8  NORMAL 
Exif.Fujifilm.Sharpness                      Short       1  Normal
Exif.Fujifilm.WhiteBalance                   Short       1  Auto
Exif.Fujifilm.Color                          Short       1  Normal
Exif.Fujifilm.FlashMode                      Short       1  Off
Exif.Fujifilm.FlashStrength                  SRational   1  0/100
Exif.Fujifilm.Macro                          Short       1  Off
Exif.Fujifilm.FocusMode                      Short       1  Auto
Exif.Fujifilm.SlowSync                       Short       1  Off
Exif.Fujifilm.PictureMode                    Short       1  Program AE
Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.Continuous                     Short       1  Off
Exif.Fujifilm.SequenceNumber                 Short       1  0
Exif.Fujifilm.BlurWarning                    Short       1  Off
Exif.Fujifilm.FocusWarning                   Short       1  Off
Exif.Fujifilm.ExposureWarning                Short       1  Off
Exif.Fujifilm.DynamicRange                   Short       1  Standard
Exif.Fujifilm.FilmMode                       Short       1  Pro Neg. Hi
Exif.Fujifilm.DynamicRangeSetting            Short       1  Auto (100-400%)
Exif.Fujifilm.Rating                         Long        1  0
""",
    """Exif.Fujifilm.Version                        Undefined   4  48 49 51 48
Exif.Fujifilm.SerialNumber                   Ascii      48  FFDT22794526     593332303134151113535030217060
Exif.Fujifilm.Quality                        Ascii       8  NORMAL 
Exif.Fujifilm.Sharpness                      Short       1  Normal
Exif.Fujifilm.WhiteBalance                   Short       1  Auto
Exif.Fujifilm.Color                          Short       1  Monochrome
Exif.Fujifilm.FlashMode                      Short       1  Off
Exif.Fujifilm.FlashStrength                  SRational   1  0/100
Exif.Fujifilm.Macro                          Short       1  Off
Exif.Fujifilm.FocusMode                      Short       1  Manual
Exif.Fujifilm.SlowSync                       Short       1  Off
Exif.Fujifilm.PictureMode                    Short       1  Program AE
Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.Continuous                     Short       1  Off
Exif.Fujifilm.SequenceNumber                 Short       1  0
Exif.Fujifilm.BlurWarning                    Short       1  On
Exif.Fujifilm.FocusWarning                   Short       1  Off
Exif.Fujifilm.ExposureWarning                Short       1  On
Exif.Fujifilm.DynamicRange                   Short       1  Standard
Exif.Fujifilm.DynamicRangeSetting            Short       1  Auto (100-400%)
Exif.Fujifilm.Rating                         Long        1  0
""",
    """Exif.Fujifilm.Version                        Undefined   4  48 49 51 48
Exif.Fujifilm.SerialNumber                   Ascii      48  FFDT22794526     593332303134151113535030217060
Exif.Fujifilm.Quality                        Ascii       8  NORMAL 
Exif.Fujifilm.Sharpness                      Short       1  Normal
Exif.Fujifilm.WhiteBalance                   Short       1  Auto
Exif.Fujifilm.Color                          Short       1  Monochrome + Ye Filter
Exif.Fujifilm.FlashMode                      Short       1  Off
Exif.Fujifilm.FlashStrength                  SRational   1  0/100
Exif.Fujifilm.Macro                          Short       1  Off
Exif.Fujifilm.FocusMode                      Short       1  Manual
Exif.Fujifilm.SlowSync                       Short       1  Off
Exif.Fujifilm.PictureMode                    Short       1  Program AE
Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.Continuous                     Short       1  Off
Exif.Fujifilm.SequenceNumber                 Short       1  0
Exif.Fujifilm.BlurWarning                    Short       1  On
Exif.Fujifilm.FocusWarning                   Short       1  Off
Exif.Fujifilm.ExposureWarning                Short       1  On
Exif.Fujifilm.DynamicRange                   Short       1  Standard
Exif.Fujifilm.DynamicRangeSetting            Short       1  Auto (100-400%)
Exif.Fujifilm.Rating                         Long        1  0
""",
    """Exif.Fujifilm.Version                        Undefined   4  48 49 51 48
Exif.Fujifilm.SerialNumber                   Ascii      48  FFDT22794526     593332303134151113535030217060
Exif.Fujifilm.Quality                        Ascii       8  NORMAL 
Exif.Fujifilm.Sharpness                      Short       1  Normal
Exif.Fujifilm.WhiteBalance                   Short       1  Auto
Exif.Fujifilm.Color                          Short       1  Monochrome + R Filter
Exif.Fujifilm.FlashMode                      Short       1  Off
Exif.Fujifilm.FlashStrength                  SRational   1  0/100
Exif.Fujifilm.Macro                          Short       1  Off
Exif.Fujifilm.FocusMode                      Short       1  Manual
Exif.Fujifilm.SlowSync                       Short       1  Off
Exif.Fujifilm.PictureMode                    Short       1  Shutter speed priority AE
Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.Continuous                     Short       1  Off
Exif.Fujifilm.SequenceNumber                 Short       1  0
Exif.Fujifilm.BlurWarning                    Short       1  Off
Exif.Fujifilm.FocusWarning                   Short       1  Off
Exif.Fujifilm.ExposureWarning                Short       1  On
Exif.Fujifilm.DynamicRange                   Short       1  Standard
Exif.Fujifilm.DynamicRangeSetting            Short       1  Auto (100-400%)
Exif.Fujifilm.Rating                         Long        1  0
""",
    """Exif.Fujifilm.Version                        Undefined   4  48 49 51 48
Exif.Fujifilm.SerialNumber                   Ascii      48  FFDT22794526     593332303134151113535030217060
Exif.Fujifilm.Quality                        Ascii       8  NORMAL 
Exif.Fujifilm.Sharpness                      Short       1  Normal
Exif.Fujifilm.WhiteBalance                   Short       1  Auto
Exif.Fujifilm.Color                          Short       1  Monochrome + G Filter
Exif.Fujifilm.FlashMode                      Short       1  Off
Exif.Fujifilm.FlashStrength                  SRational   1  0/100
Exif.Fujifilm.Macro                          Short       1  Off
Exif.Fujifilm.FocusMode                      Short       1  Manual
Exif.Fujifilm.SlowSync                       Short       1  Off
Exif.Fujifilm.PictureMode                    Short       1  Shutter speed priority AE
Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.Continuous                     Short       1  Off
Exif.Fujifilm.SequenceNumber                 Short       1  0
Exif.Fujifilm.BlurWarning                    Short       1  Off
Exif.Fujifilm.FocusWarning                   Short       1  Off
Exif.Fujifilm.ExposureWarning                Short       1  On
Exif.Fujifilm.DynamicRange                   Short       1  Standard
Exif.Fujifilm.DynamicRangeSetting            Short       1  Auto (100-400%)
Exif.Fujifilm.Rating                         Long        1  0
""",
    """Exif.Fujifilm.Version                        Undefined   4  48 49 51 48
Exif.Fujifilm.SerialNumber                   Ascii      48  FFDT22794526     593332303134151113535030217060
Exif.Fujifilm.Quality                        Ascii       8  NORMAL 
Exif.Fujifilm.Sharpness                      Short       1  Normal
Exif.Fujifilm.WhiteBalance                   Short       1  Auto
Exif.Fujifilm.Color                          Short       1  Sepia
Exif.Fujifilm.FlashMode                      Short       1  Off
Exif.Fujifilm.FlashStrength                  SRational   1  0/100
Exif.Fujifilm.Macro                          Short       1  Off
Exif.Fujifilm.FocusMode                      Short       1  Manual
Exif.Fujifilm.SlowSync                       Short       1  Off
Exif.Fujifilm.PictureMode                    Short       1  Shutter speed priority AE
Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.Continuous                     Short       1  Off
Exif.Fujifilm.SequenceNumber                 Short       1  0
Exif.Fujifilm.BlurWarning                    Short       1  Off
Exif.Fujifilm.FocusWarning                   Short       1  Off
Exif.Fujifilm.ExposureWarning                Short       1  On
Exif.Fujifilm.DynamicRange                   Short       1  Standard
Exif.Fujifilm.DynamicRangeSetting            Short       1  Auto (100-400%)
Exif.Fujifilm.Rating                         Long        1  0
"""
    ]
    stderr = [""] * 10
    retval = [0] * 10
